#!/usr/bin/python
# coding:utf-8
hb_ip = '192.168.223.152'

sql_user_hb = {
    'ip': hb_ip,
    'user': 'readonly',
    'pass': 'c24mg2e6',
    'port': '3306'
}

sql_write_path_hb = {
    'commodities':
        'mysql+mysqldb://%s:%s@%s:%s/commodities?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
    'stocks':
        'mysql+mysqldb://%s:%s@%s:%s/stocks?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
    'work':
        'mysql+mysqldb://%s:%s@%s:%s/work?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
    'daily':
        'mysql+mysqldb://%s:%s@%s:%s/daily_data?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
}

index_underlying = {
    'IC': '中证500',
    'IF': '沪深300',
    'IH': '上证50'
}
underlying_code = {
    'IC': '000905',
    'IF': '000300',
    'IH': '000015',
}

# 交易所代码
HSJY_EXCHANGE_SHFE = 10
HSJY_EXCHANGE_INE = 11
HSJY_EXCHANGE_DCE = 13
HSJY_EXCHANGE_CZCE = 15
HSJY_EXCHANGE_CFFEX = 20

exchange_list = [
    HSJY_EXCHANGE_CFFEX,
    HSJY_EXCHANGE_DCE,
    HSJY_EXCHANGE_INE,
    HSJY_EXCHANGE_CZCE,
    HSJY_EXCHANGE_SHFE
]

exchange_dict = {
    HSJY_EXCHANGE_CFFEX: '中金所',
    HSJY_EXCHANGE_DCE: '大商所',
    HSJY_EXCHANGE_INE: '能源所',
    HSJY_EXCHANGE_CZCE: '郑商所',
    HSJY_EXCHANGE_SHFE: '上期所'
}

# 品种代码
HSJY_PRODUCT_CODE = {
    'IH': 46,
    'IF': 3145,
    'IC': 4978
}

# 虚拟合约代码
HSJY_VIRTUAL_CONTRACT = {
    'FINANCE': {
        'IF': {
            '当月': 2000440,
            '次月': 2000441,
            '当季': 2000442,
            '次季': 2000443,
        },
        'IC': {
            '当月': 2006539,
            '次月': 2006540,
            '当季': 2006541,
            '次季': 2006542,
        },
        'IH': {
            '当月': 2006535,
            '次月': 2006536,
            '当季': 2006537,
            '次季': 2006538,
        },
    }
}

HSJY_PRODUCT_TABLE = 'Fut_FuturesContract'
HSJY_CONTRACT_TABLE = 'Fut_ContractMain'

# Fut_FuturesContract常用字段
properties_product = [
    'ID',
    'ContractName'.upper(),
    'TradingCode'.upper(),
    # 'ContractOption'.upper(),
    'Exchange'.upper(),
    'ContractMulti'.upper(),
    'PriceUnit'.upper(),
    'LittlestChangeUnit'.upper(),
    'ContractInnerCode'.upper(),
]

# Fut_ContractMain常用字段
properties_contract = [
    'ID',
    'ContractInnerCode'.upper(),
    'ContractCode'.upper(),
    'ContractName'.upper(),
    'ExchangeCode'.upper(),
    'ContractMultiplier'.upper(),
    'PriceUnit'.upper(),
    'LittlestChangeUnit'.upper(),
    'DeliveryYear'.upper(),
    'DeliveryMonth'.upper(),
    'LastTradingDate'.upper(),
    # 'OptionCode'.upper(),
    'VarietyInnerCode'.upper(),
]

# FUT_DAILYQUOTE 常用字段，商品期货
properties_com_k = [
    'ID',
    'InnerCode'.upper(),
    'EndDate'.upper(),
    # 'DATE_ADD(EndDate, INTERVAL -8 HOUR) as date'.upper(),
    'ReportPeriod'.upper(),
    'Exchange'.upper(),
    'ContractName'.upper(),
    'SettlementYear'.upper(),
    'SettlementMonth'.upper(),
    'OpenPrice'.upper(),
    'HighPrice'.upper(),
    'LowPrice'.upper(),
    'ClosePrice'.upper(),
    'SettlePrice'.upper(),
    'Volume'.upper(),
    'OpenInterest'.upper(),
    'OpenInterestChange'.upper(),
    'Turnover'.upper(),
]

# FUT_TRADINGQUOTE 常用字段，金融期货
properties_fin_k = [
    'ID',
    'ContractInnerCode'.upper(),
    'TradingDay'.upper(),
    # 'DATE_ADD(EndDate, INTERVAL -8 HOUR) as date'.upper(),
    # 'ReportPeriod'.upper(),
    'OptionCode'.upper(),
    'ExchangeCode'.upper(),
    'ContractCode'.upper(),
    # 'SettlementYear'.upper(),
    # 'SettlementMonth'.upper(),
    'OpenPrice'.upper(),
    'HighPrice'.upper(),
    'LowPrice'.upper(),
    'ClosePrice'.upper(),
    'SettlePrice'.upper(),
    'TurnoverVolume'.upper(),  # 成交量
    'TurnoverValue'.upper(),  # 成交额
    'OpenInterest'.upper(),  # 持仓量
    'ChangePCTOpenInterest'.upper(),
]


# FINCHINA.CHDQUOTE 字段，股票
properties_stk_k = [
    'TDATE',
    'EXCHANGE',
    'SYMBOL',
    'SNAME',
    'LCLOSE',
    'TOPEN',
    'TCLOSE',
    'HIGH',
    'LOW',
    'VOTURNOVER',
    'VATURNOVER',
    'CHG',
    'PCHG',
    'MCAP',
    'TCAP',
    'TURNOVER',
]


db = 'readonly'
db_tables = {
    'hsjy_com_daily_quote': 'HSJY_GG.Fut_DailyQuote'.upper(),
    'hsjy_fin_daily_quote': 'HSJY_GG.Fut_TradingQuote'.upper(),
    'hsjy_fut_contract_info': 'HSJY_GG.Fut_ContractMain'.upper(),
    'hsjy_cons_info': 'HSJY_GG.CT_Product'.upper(),
    'hsjy_product_info': 'HSJY_GG.Fut_FuturesContract'.upper(),
    'hsjy_wr': 'HSJY_GG.Fut_WRStatByOption'.upper(),
    'hsjy_member_rank': 'HSJY_GG.FUT_MEMBERRANKBYCONTRACT',
    'ch_stocks_daily_quote': 'FINCHINA.CHDQUOTE',
    'index_daily_quote': 'FUNDDB.ZSJY',
    'stocks_daily_quote': 'FUNDDB.ZGJY',
}

wind_sec = {
    'CCFI.WI': 'Wind商品指数',
    'NMFI.WI': 'Wind贵金属',
    'NFFI.WI': 'Wind有色',
    'JJRI.WI': 'Wind煤焦钢矿',
    'NMBM.WI': 'Wind非金属建材',
    'ENFI.WI': 'Wind能源',
    'CIFI.WI': 'Wind化工',
    'CRFI.WI': 'Wind谷物',
    'OOFI.WI': 'Wind油脂油料',
    'SOFI.WI': 'Wind软商品',
    'APFI.WI': 'Wind农副产品'
}



#!/usr/bin/python
# coding:utf-8
from hbshare.quant.CChen.func import generate_table
import pandas as pd
import numpy as np
import pymysql
import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter
from pandas.plotting import register_matplotlib_converters
from datetime import datetime

register_matplotlib_converters()

sql_l = '''(
    `ID` bigint not null AUTO_INCREMENT,
    `TDATE`  date not null,
    `EXCHANGE`  int,
    `PCODE`  int,
    `UCODE`  int,
    `CLOSE0`  float,
    `CLOSE` float,
    `CLOSE1` float,
    `POS` int,
    `WEIGHT`  double,
    `RETURN` double,
    `FACTORVALUE` double,
    `FACTOR`  varchar(255),
    primary key (`ID`)
    )
    '''

sql_index = '''(
    `ID` bigint not null AUTO_INCREMENT,
    `TDATE`  date not null,
    `CLOSE` float,
    `FACTOR`  varchar(255),
    primary key (`ID`)
    )
    '''


def factor_table_gen(table, sql_info):

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_l,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='CTA factor based on hsjy data'
    )
    print(table + ' generated')


def index_table_gen(table, sql_info):

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_index,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='CTA INDEX'
    )
    print(table + ' generated')


def index_gen(sql_path, sql_info, table_factor='cta_factor', table_index='cta_index'):
    index_table_gen(table=table_index, sql_info=sql_info)
    end_date_all = pd.read_sql_query(
        'select distinct TDATE from ' + table_factor + ' order by TDATE desc limit 1', sql_path
    )
    factors = pd.read_sql_query('select distinct FACTOR from ' + table_factor, sql_path)
    # factors = pd.read_sql_query(
    #     'select FACTOR from ' + table_factor
    #     + ' where TDATE<=' + end_date_all['TDATE'][0].strftime('%Y%m%d')
    #     + ' group by FACTOR',
    #     sql_path
    # )
    if len(factors) == 0:
        print('No factor return')
        return
    else:
        for i in range(len(factors['FACTOR'])):
            # factor = 'xswr_d1_q75'
            factor = factors['FACTOR'][i]
            last_factor_index = pd.read_sql_query(
                'select * from ' + table_index
                + ' where TDATE<=' + end_date_all['TDATE'][0].strftime('%Y%m%d')
                + ' and FACTOR=\'' + factor
                + '\' order by TDATE desc limit 1',
                sql_path
            )
            if len(last_factor_index) == 0:
                date0 = datetime(1990, 1, 1).date()
                index0 = 1000
            else:
                date0 = last_factor_index['TDATE'][0]
                index0 = last_factor_index['CLOSE'][0]

            print(
                str(i + 1) + '/' + str(len(factors['FACTOR'])) + ', '
                + factor + ', start date: ' + date0.strftime('%Y-%m-%d')
            )

            if date0 == end_date_all['TDATE'][0]:
                print('\tNo new date, ' + date0.strftime('%Y-%m-%d'))
                continue

            factor_data = pd.read_sql_query(
                'select * from ' + table_factor + ' where TDATE>' + date0.strftime('%Y%m%d')
                + ' and FACTOR=\'' + factor + '\' order by TDATE',
                sql_path
            )
            if len(factor_data) > 0:
                factor_data['RETURN'] = factor_data['RETURN'] * factor_data['WEIGHT'] * factor_data['POS']
                factor_data_g = factor_data.groupby(by='TDATE').sum()[['RETURN']].reset_index()
                factor_data_g['ret'] = factor_data_g['RETURN'] / 10000 + 1.0
                factor_data_g['CLOSE'] = factor_data_g['ret'].cumprod()
                factor_data_g['CLOSE'] = factor_data_g['CLOSE'] * index0
                factor_data_g['FACTOR'] = factor
                factor_data_g[['TDATE', 'CLOSE', 'FACTOR']].to_sql(
                    table_index, sql_path, if_exists='append', index=False
                )
                print(
                    '\t' + str(len(factor_data_g)) + ', to sql, '
                    + factor_data_g['TDATE'][len(factor_data_g) - 1].strftime('%Y-%m-%d')
                )
            else:
                print('\tNo new data')


def get_last_position(table, before_date, factor, sql_path):
    existing_cal = pd.read_sql_query(
        'select distinct `TDATE` from ' + table + ' where `TDATE`<=' + before_date.strftime('%Y%m%d')
        + ' and `FACTOR`=\'' + factor + '\' order by `TDATE` desc limit 1',
        sql_path
    )
    if len(existing_cal) >= 1:
        pos = pd.read_sql_query(
            'select * from ' + table + ' where `TDATE`=' + existing_cal['TDATE'][0].strftime('%Y%m%d')
            + ' and `FACTOR`=\'' + factor + '\'',
            sql_path
        )
        print(factor + ', load last date: ' + existing_cal['TDATE'][0].strftime('%Y-%m-%d'))
        return pos

    else:
        print('load last date: None')
        return None


def load_factor(sql_path, factor_name, table='cta_factor'):
    index = factor_name

    data = pd.read_sql_query(
        'select * from ' + table + ' where FACTOR=\'' + index + '\'',
        sql_path
    )
    return data


def factor_stats(data, trade_days=250, show=False):
    data['RETURN'] = data['RETURN'] * data['WEIGHT'] * data['POS']
    data_g = data.groupby(by='TDATE').sum()[['RETURN']].reset_index()
    # data_g = data.groupby(by='TDATE').mean()[['RETURN']].reset_index()
    data_g['ret'] = data_g['RETURN'] / 10000 + 1
    data_g['ind'] = data_g['ret'].cumprod()
    data_g['max_ind'] = data_g['ind'].cummax()
    cumulative_return = data_g['ind'][len(data_g) - 1] / data_g['ind'][0] - 1.0
    annualized_return = (cumulative_return + 1.0) ** (
            365 / (data_g['TDATE'][len(data_g) - 1] - data_g['TDATE'][0]).days
    ) - 1
    return_sample = ((data_g['ind'] - data_g['ind'].shift(1)) / data_g['ind'].shift(1))[1:]
    annualized_return_average = (return_sample.mean() + 1) ** trade_days - 1
    sigma = return_sample.std(ddof=1) * np.sqrt(trade_days)

    data_g['dd'] = (data_g['ind'] - data_g['max_ind']) / data_g['max_ind']
    data_g['year'] = data_g['TDATE'].apply(lambda x: x.year)

    range_dates = [data_g['TDATE'][0]]

    year_slice = data_g.drop_duplicates(subset=['year'], keep='last')

    if len(year_slice) > 0:
        range_dates += year_slice['TDATE'].tolist()

    result = {
        '累计收益': cumulative_return,
        '最大回撤': min(data_g['dd'][1:]),
        '年化收益率（头尾）': annualized_return,
        '年化收益率（平均）': annualized_return_average,
        '波动率': sigma
    }

    if show:
        print(
            '累计收益： \n\t%.2f%%' % round(cumulative_return * 100, 2)
        )
        print(
            '最大回撤：      \n\t%.2f%%' % round(min(data_g['dd'][1:]) * 100, 2)
        )
        print(
            '年化收益率（头尾）： \n\t%.2f%%' % round(annualized_return * 100, 2)
        )
        print(
            '年化收益率（平均）： \n\t%.2f%%' % round(annualized_return_average * 100, 2)
        )
        print(
            '波动率： \n\t%.2f%%' % round(sigma * 100, 2)
        )
        print(
            '收益波动比： \n\t%.2f' % round(annualized_return_average / sigma, 2)
        )
        if min(data_g['dd'][1:]) != 0:
            print(
                '收益回撤比：        \n\t%.2f' % round(-annualized_return_average / min(data_g['dd'][1:]), 2)
            )
        else:
            print(
                '收益回撤比：       \n\tinf'
            )

        print('')
        for i in range(1, len(range_dates)):
            range_return = (
                                   data_g[data_g['TDATE'] == range_dates[i]]['ind'].tolist()[0]
                                   / data_g[data_g['TDATE'] == range_dates[i - 1]]['ind'].tolist()[0]
                           ) - 1
            result[range_dates[i - 1].strftime('%Y/%m/%d') + ' - ' + range_dates[i].strftime('%Y/%m/%d')] = range_return
            print(
                range_dates[i - 1].strftime('%Y/%m/%d') + ' - ' + range_dates[i].strftime('%Y/%m/%d')
                + ':\t收益：%.2f%%' % round(range_return * 100, 2)
            )
    return data_g, result


def factor_plot(data, plot_title):
    def to_percent(temp, position):
        return '%.2f' % (100 * temp) + '%'

    data_g = data
    fig = plt.figure()
    ax1 = fig.add_subplot(1, 1, 1)

    ax1.fill_between(data_g['TDATE'], data_g['dd'], color='gray', alpha=0.3)
    plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent))

    ax2 = ax1.twinx()
    ax2.plot(data_g['TDATE'], data_g['ind'])

    plt.title(plot_title)
    plt.show()


def factor_compute(
        window_days_list,
        to_sql_path,
        sql_info,
        start_date=datetime(2010, 1, 1).date(),
        end_date=datetime.now(),
        to_table='cta_factor',
        data=None,
        factor_func=None,
        **kwargs
):
    if 'min_volume' in kwargs:
        min_volume = kwargs['min_volume']
    else:
        min_volume = 10000

    factor_table_gen(table=to_table, sql_info=sql_info)

    for window_days in window_days_list:
        if 'hedge_ratio_list' in kwargs:
            for quantile in kwargs['hedge_ratio_list']:
                index_name = factor_func.__name__ + '_d' + str(window_days) + '_q' + str(quantile)
                exist_index = get_last_position(
                    table=to_table,
                    before_date=end_date,
                    factor=index_name,
                    sql_path=to_sql_path
                )
                if exist_index is not None:
                    start_date0 = exist_index['TDATE'][0]
                else:
                    start_date0 = start_date

                pos = factor_func(
                    start_date=start_date0,
                    data=data,
                    window_days=window_days,
                    quantile=quantile,
                    min_volume=min_volume,
                    **kwargs
                )
                if pos is not None:
                    pos.to_sql(to_table, to_sql_path, if_exists='append', index=False)
                    print('\t' + index_name + ' to sql, ' + pos['TDATE'].tolist()[-1].strftime('%Y-%m-%d'))
                else:
                    print('\t' + index_name + ', no new data')

        else:
            index_name = factor_func.__name__ + '_d' + str(window_days)
            exist_index = get_last_position(
                table=to_table,
                before_date=end_date,
                factor=index_name,
                sql_path=to_sql_path
            )

            if exist_index is not None:
                start_date0 = exist_index['TDATE'][0]
            else:
                start_date0 = start_date

            pos = factor_func(
                start_date=start_date0, data=data, window_days=window_days, min_volume=min_volume, **kwargs
            )
            if pos is not None:
                pos.to_sql(to_table, to_sql_path, if_exists='append', index=False)
                print('\t' + index_name + ' to sql, ' + pos['TDATE'].tolist()[-1].strftime('%Y-%m-%d'))
            else:
                print('\t' + index_name + ', no new data')


if __name__ == '__main__':
    index_gen()



# -*- coding: utf-8 -*-

from collections import namedtuple
from enum import Enum


StrategyParameterTuple = namedtuple('StrategyParameterTuple', ['PORTFOLIO_NAME', 'PORTFOLIO_EN_NAME', 'OPEN_DATE', 'DELAY', 'REFRESH_DATE', 'CONSTRUCT_TYPE'])


class TimeDateFormat(Enum):
    YMDHYPHEN = '%Y-%m-%d'
    YMD = '%Y%m%d'
    YMDHMSHYPHEN = '%Y-%m-%d %H:%M:%S'
    YMDHMS = '%Y:%m:%d %H:%M:%S'
    Y = '%Y'
    HM = '%H:%M'
    YMDCN = '%Y年%m月%d日'
    YM = '%Y%m'
    MDHYPHEN = '%m-%d'
    YMDSLASH = '%Y/%m/%d'

class CovMethod(Enum):
    SAMPLECOV = 'SAMPLECOV'
    LEDOITWOLF = 'LEDOITWOLF'


class StyleFactors(Enum):
    BETA = 'BETA'
    MOMENTUM = 'MOMENTUM'
    SIZE = 'SIZE'
    EARNYILD = 'EARNYILD'
    RESVOL = 'RESVOL'
    GROWTH = 'GROWTH'
    BTOP = 'BTOP'
    LEVERAGE = 'LEVERAGE'
    LIQUIDTY = 'LIQUIDTY'
    SIZENL = 'SIZENL'


class IndustryFactors(Enum):
    Bank = ('Bank', '银行')
    RealEstate = ('RealEstate', '房地产')
    Health = ('Health', '生物医药')
    Transportation = ('Transportation', '交通运输')
    Mining = ('Mining', '采掘')
    NonFerMetal = ('NonFerMetal', '有色金属')
    HouseApp = ('HouseApp', '家用电器')
    LeiService = ('LeiService', '休闲服务')
    MachiEquip = ('MachiEquip', '机械设备')
    BuildDeco = ('BuildDeco', '建筑装饰')
    CommeTrade = ('CommeTrade', '商业贸易')
    CONMAT = ('CONMAT', '建筑材料')
    Auto = ('Auto', '汽车')
    Textile = ('Textile', '纺织服装')
    FoodBever = ('FoodBever', '食品饮料')
    Electronics = ('Electronics', '电子')
    Computer = ('computer', '计算机')
    LightIndus = ('LightIndus', '轻工制造')
    Utilities = ('Utilities', '公共事业')
    Telecom = ('Telecom', '通信')
    AgriForest = ('AgriForest', '农林牧渔')
    CHEM = ('CHEM', '化工')
    Media = ('Media', '传媒')
    IronSteel = ('IronSteel', '钢铁')
    NonBankFinan = ('NonBankFinan', '非银金融')
    ELECEQP = ('ELECEQP', '电气设备')
    AERODEF = ('AERODEF', '国防军工')
    Conglomerates = ('Conglomerates', '综合')


class Freq(Enum):
    DAY = 'DAY'
    MONTH = 'MONTH'
    WEEK = 'WEEK'


class ProcessType(Enum):
    BATCH = 'BATCH'
    ONLINE = 'ONLINE'

class HoldType(Enum):
    SHORT = 'SHORT'
    LONG = 'LONG'

class UploadType(Enum):
    FEEDS = 'FEEDS'
    STRATEGY = 'STRATEGY'

class StrategyParameters(Enum):
    ANALYSTADJ1 = StrategyParameterTuple(PORTFOLIO_NAME='王牌掘金一号策略', PORTFOLIO_EN_NAME='ANALYSTADJ1', OPEN_DATE='2017-01-01', DELAY=1, REFRESH_DATE=1, CONSTRUCT_TYPE='OPEN')








"""
Wind计算的转债的隐含波动率
"""
from hbshare.fe.common.util.data_loader import get_trading_day_list
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
import pandas as pd
from tqdm import tqdm
from datetime import datetime
import hbshare as hbs
from WindPy import w

w.start()


class CBMarketIV:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = "cb_market_iv"
        self._load_data()

    def _load_data(self):
        trading_day_list = get_trading_day_list(self.start_date, self.end_date)
        # data set
        res = w.wset("cbissue", "startdate=2012-01-01;enddate={}".format(datetime.now().strftime('%Y-%m-%d')))
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("Fetching cbissue data error!")
        else:
            data = pd.DataFrame(res.Data, index=res.Fields, columns=res.Codes).T
        # preprocess
        issue_data = data[data['issue_type'].isin(['优先配售和上网定价', '优先配售,网上定价和网下配售'])]
        issue_data = issue_data[['bond_code', 'bond_name', 'listing_date', 'interest_end_date']].dropna()
        issue_data['listing_date'] = issue_data['listing_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        issue_data['interest_end_date'] = issue_data['interest_end_date'].apply(
            lambda x: datetime.strftime(x, '%Y%m%d'))
        # iv data
        iv_list = []
        for date in tqdm(trading_day_list):
            exist_cb = issue_data[(issue_data['listing_date'] < date) & (issue_data['interest_end_date'] > date)]
            cb_code_list = exist_cb['bond_code'].unique()
            res = w.wss(','.join(cb_code_list), "impliedvol", "tradeDate={};rfIndex=7".format(date))
            if res.ErrorCode != 0:
                data = pd.DataFrame()
                print("fetch implied volatility data error: trade_date = {}".format(date))
            else:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times)
            data.columns = ['iv']
            data.index.name = 'bond_code'
            data = data.reset_index().dropna()
            data['trade_date'] = date
            iv_list.append(data)

        iv_df = pd.concat(iv_list)
        iv_df['ticker'] = iv_df['bond_code'].apply(lambda x: x.split('.')[0])
        iv_df['iv'] = iv_df['iv'].round(4)
        self.iv_df = iv_df[['trade_date', 'ticker', 'iv']]

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.iv_df
            sql_script = "delete from {} where trade_date >= {} and trade_date <= {}".format(
                self.table_name, self.start_date, self.end_date)
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table cb_market_iv(
                id int auto_increment primary key,
                trade_date date not null,
                ticker varchar(20),
                iv decimal(7, 4)) 
            """
            create_table(self.table_name, sql_script)
            data = self.iv_df
            WriteToDB().write_to_db(data, self.table_name)


class CBMarketPrice:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = "cb_market_price"
        self._load_data()

    def _load_data(self):
        trading_day_list = get_trading_day_list(self.start_date, self.end_date)
        # data set
        res = w.wset("cbissue", "startdate=2012-01-01;enddate={}".format(datetime.now().strftime('%Y-%m-%d')))
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("Fetching cbissue data error!")
        else:
            data = pd.DataFrame(res.Data, index=res.Fields, columns=res.Codes).T
        # preprocess
        issue_data = data[data['issue_type'].isin(['优先配售和上网定价', '优先配售,网上定价和网下配售'])]
        issue_data = issue_data[['bond_code', 'bond_name', 'listing_date', 'interest_end_date']].dropna()
        issue_data['listing_date'] = issue_data['listing_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        issue_data['interest_end_date'] = issue_data['interest_end_date'].apply(
            lambda x: datetime.strftime(x, '%Y%m%d'))
        # price data
        price_list = []
        for date in tqdm(trading_day_list):
            exist_cb = issue_data[(issue_data['listing_date'] < date) & (issue_data['interest_end_date'] > date)]
            cb_code_list = exist_cb['bond_code'].unique()
            # convprice
            res = w.wss(','.join(cb_code_list), 'convprice,underlyingcode', "tradeDate={}".format(date))
            if res.ErrorCode != 0:
                data = pd.DataFrame()
                print("fetch convprice data error: trade_date = {}".format(date))
            else:
                data = pd.DataFrame(data=res.Data, index=res.Fields, columns=res.Codes).T
            data.columns = ['convprice', 'sec_code']
            data.index.name = 'cb_code'
            conv_price = data.reset_index().dropna()
            conv_price['ticker'] = conv_price['sec_code'].apply(lambda x: x.split('.')[0])
            # close price
            sql_script = "SELECT SYMBOL, TDATE, TCLOSE FROM finchina.CHDQUOTE WHERE" \
                         " TDATE = {}".format(date)
            data = pd.DataFrame(hbs.db_data_query("readonly", sql_script, page_size=5000)['data']).rename(
                columns={"SYMBOL": "ticker", "TCLOSE": "stock_price"})
            # merge
            data = pd.merge(conv_price, data[['ticker', 'stock_price']], on='ticker')
            data['trade_date'] = date
            price_list.append(data)

        price_df = pd.concat(price_list)
        price_df['ticker'] = price_df['cb_code'].apply(lambda x: x.split('.')[0])
        self.price_df = price_df[['trade_date', 'ticker', 'convprice', 'stock_price']]

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.price_df
            sql_script = "delete from {} where trade_date >= {} and trade_date <= {}".format(
                self.table_name, self.start_date, self.end_date)
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table cb_market_price(
                id int auto_increment primary key,
                trade_date date not null,
                ticker varchar(20),
                convprice decimal(6, 2),
                stock_price decimal(6, 2)) 
            """
            create_table(self.table_name, sql_script)
            data = self.price_df
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    # CBMarketIV('20180101', '20201231').get_construct_result()
    CBMarketPrice('20200101', '20201231', is_increment=1).get_construct_result()


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.utils.logger_utils import LoggerUtils
from statsmodels.tsa.filters.hp_filter import hpfilter
from statsmodels.tsa.seasonal import seasonal_decompose
import pandas as pd


class DataProcessor(object):
    def __init__(self):
        self.freq_map = {
            'day': 'D',
            'week': 'W',
            'month': 'M',
            'season': 'Q',
            'annual': 'A'
        }

    def _sample_data(self, signal_series, frequency, method='last'):
        """
        method = 'last' 默认sample每期的最后一个点，
        method = 'first' sample每期的初始点
        method = 'mean' sample平均值。
        :param signal_series:
        :param frequency:
        :param method:
        :return:
        """
        if frequency not in self.freq_map:
            raise ValueError('frequency not in {}'.format(self.freq_map.keys()))

        if method is None:
            method = 'last'

        reindex_freq = self.freq_map[frequency]

        signal_series.index = pd.to_datetime(signal_series.index).to_period(reindex_freq)

        if method == 'last':
            signal_series = signal_series.groupby(signal_series.index).last()
        elif method == 'first':
            signal_series = signal_series.groupby(signal_series.index).first()
        elif method == 'mean':
            signal_series = signal_series.groupby(signal_series.index).mean()
        elif method == 'sum':
            signal_series = signal_series.groupby(signal_series.index).sum()
        else:
            raise ValueError("method unknown {}".format(method))

        begin_date = str(signal_series.index.min())
        end_date = str(signal_series.index.max())
        period_list = pd.period_range(begin_date, end_date, freq=reindex_freq)
        signal_series = signal_series.reindex(period_list)

        signal_series.index = signal_series.index.strftime('%Y%m%d')

        signal_series.index = pd.to_datetime(signal_series.index)
        return signal_series

    def _moving_arerage(self, signal_series, window):
        return signal_series.rolling(window).mean().dropna()

    def _time_index_month_unique(self,signal_series):
        signal_series.index = pd.to_datetime(signal_series.index).to_period('M')
        return signal_series

    def _reindex_and_interpolate(self, signal_series,frequency):

        signal_series = self._sample_data(signal_series. frequency)

        ori_index = signal_series.index

        signal_series.index = [i for i in range(len(signal_series))]

        signal_series = signal_series.interpolate(mothed='pchip')

        signal_series.index = ori_index

        return signal_series.dropna()

    def _make_trend_cycle(self, signal_series, freq=12):
        # se-idx: contain a couple of indices indicating macroeconomic state
        # remove irregularities and seasonal fluctuations
        decomp = seasonal_decompose(signal_series, model='additive', freq=freq, two_sided=False)
        trend_cycle = decomp.resid + decomp.trend
        return trend_cycle

    def _seasonal_adjest(self, signal_series, window_size):
        def _make_hycycles(se_idx, freq=12):
            # se_idx: contain a couple of indices indicating macroeconomic state
            # remove irregularities and seasonal fluctuations
            decomp = seasonal_decompose(se_idx, model='additive', freq=freq, two_sided=False)
            irreg = decomp.seasonal
            trend_cycle = se_idx - irreg
            return trend_cycle[-1]

        rolling_adj = signal_series.rolling(window=window_size).apply(_make_hycycles)
        head_tc = self._make_trend_cycle(signal_series.head(window_size))

        rolling_adj[rolling_adj.isnull()] = head_tc.reindex(rolling_adj[rolling_adj.isnull()].index)

        return rolling_adj.dropna()

    def _filter_trend(self, signal_series):
        cycle, trend = hpfilter(signal_series.dropna(), lamb=129600)
        return cycle

    def _cumsum_by_year(self, signal_series):
        year = pd.to_datetime(signal_series.index).year
        signal_series = signal_series.groupby(year).cumsum()
        return signal_series

    def _standard(self, signal_series):
        signal_series = (signal_series - signal_series.mean()) / signal_series.std()
        return signal_series

    def _min_max(self, signal_series):
        signal_series = signal_series.divide(signal_series.abs().max())
        return signal_series

    def _ttm(self, signal_series):
        return signal_series.rolling(12).mean().dropna()

    def _mom(self, signal_series, frequency):
        if frequency == 'm':
            return signal_series.pct_change(1).dropna()
        if frequency == 'd':
            return signal_series.pct_chage(20).dropna()

    def _yoy(self, signal_series):
        return signal_series.pct_change(12).dropna()

    def _ffill(self, signal_series):
        return signal_series.fillna(method='ffill').dropna()

    def _diff(self, signal_series):
        return signal_series.diff(1).dropna()

    def _binary(self, signal_series):
        signal_series[signal_series > 0] = 1
        signal_series[signal_series <= 0] = 0
        return signal_series

    def work(self, signal_series, flow):
        if isinstance(flow, dict):
            if flow['name'] == 'sample':
                signal_series = self._sample_data(signal_series, frequency=flow['frequency'], method=flow.get('method'))
            elif flow['name'] == 'unique_time':
                pass
                # signal_series = self._time_index_month_unique(signal_series)
            elif flow['name'] == 'ma':
                signal_series = self._moving_arerage(signal_series, flow['window'])
            elif flow['name'] == 'interpolate':
                signal_series = self._reindex_and_interpolate(signal_series, frequency=flow['frequency'])
            elif flow['name'] == 'seasonal_adjust':
                signal_series = self._seasonal_adjest(signal_series, window_size=flow['window_size'])
            elif flow['name'] == 'filter_trend':
                signal_series = self._filter_trend(signal_series)
            elif flow['name'] == 'standard':
                signal_series = self._standard(signal_series)
            elif flow['name'] == 'ttm':
                signal_series = self._moving_arerage(signal_series, flow['window'])
            elif flow['name'] == 'min_max':
                signal_series = self._min_max(signal_series)
            elif flow['name'] == 'mom':
                signal_series = self._mom(signal_series, frequency=flow['frequency'])
            elif flow['name'] == 'yoy':
                signal_series = self._yoy(signal_series)
            elif flow['name'] == 'diff':
                signal_series = self._diff(signal_series)
            elif flow['name'] == 'binary':
                signal_series = self._binary(signal_series)
            elif flow['name'] == 'ffill':
                signal_series = self._ffill(signal_series)
            elif flow['name'] == 'year_cumsum':
                signal_series = self._cumsum_by_year(signal_series)
            elif flow['name'] == 'miuus':
                signal_series = self._minus(signal_series, flow['other_indic_id'], flow['start_date'])
            else:
                LoggerUtils.get_logger().info('do not recognize flow {}'.format(flow['name']))
        else:
            LoggerUtils.get_logger().info('flow is not a dict')
        return signal_series

    def process(self, signal_series, work_flow):
        if work_flow is None:
            return signal_series
        for flow in work_flow:
            signal_series = self.work(signal_series, flow)
        return signal_series




"""
公募基金的Brinson持仓归因模型-dateyes版本
"""
import pandas as pd
from hbshare.fe.common.util.logger import logger
from hbshare.fe.common.util.data_loader import HoldingAttributionLoader
from hbshare.fe.common.util.verifier import verify_type, verify_trading_day, \
    verify_df_not_null, verify_simple_data_duplicate
from hbshare.fe.common.util.helper_func import \
    compute_attr_linking_coef, accumulate_timing_return, accumulate_management_return


class EquityBrinsonAttributionOutputs:
    def __init__(self):
        self.brinson_attr_series = None
        self.sector_attr_dict = None
        self.accumulate_summary = None


class EquityBrinsonAttribution:
    def __init__(self, fund_id, benchmark_id, start_date, end_date):
        """
        @param fund_id: 公募基金代码
        @param benchmark_id: 基准指数代码
        @param start_date: 起始日期
        @param end_date: 结束日期
        """
        self.fund_id = fund_id
        self.benchmark_id = benchmark_id
        self.start_date = start_date
        self.end_date = end_date
        # result init (without risk)
        self.active_weight = None
        self.active_return = None
        self.total_return_df = None

        self._verify_input_param()
        self._load_data()
        self._verify()
        self._init_data_param()

    def _verify_input_param(self):
        verify_type(self.fund_id, 'fund_id', str)
        verify_type(self.benchmark_id, 'benchmark_id', str)
        verify_type(self.start_date, 'start_date', str)
        verify_type(self.end_date, 'end_date', str)

    def _load_data(self):
        self.data_param = HoldingAttributionLoader(
            self.fund_id, self.benchmark_id, self.start_date, self.end_date, mode='brinson').load()

    def _verify(self):
        if not self.data_param:
            return
        trading_day_list = self.data_param.get('trading_day_list')
        portfolio_weight_series_dict = self.data_param.get('portfolio_weight_series_dict')
        benchmark_weight_series_dict = self.data_param.get('benchmark_weight_series_dict')
        security_return_series_dict = self.data_param.get('security_return_series_dict')
        security_sector_series_dict = self.data_param.get('security_sector_series_dict')
        nav_active_return_dict = self.data_param.get('nav_active_return_dict')
        verify_trading_day(trading_day_list)
        verify_type(portfolio_weight_series_dict, 'portfolio_weight_series_dict', dict)
        verify_type(benchmark_weight_series_dict, 'benchmark_weight_series_dict', dict)
        verify_type(security_return_series_dict, 'security_return_series_dict', dict)
        verify_type(security_sector_series_dict, 'security_sector_series_dict', dict)
        verify_type(nav_active_return_dict, 'nav_active_return_dict', dict)

        for i in range(len(trading_day_list) - 1):
            trading_day = trading_day_list[i]
            next_trading_day = trading_day_list[i + 1]

            portfolio_weight_series = portfolio_weight_series_dict.get(trading_day)
            verify_type(portfolio_weight_series, 'portfolio_weight_series', pd.Series, trading_day)
            verify_df_not_null(portfolio_weight_series, 'portfolio_weight_series', trading_day)
            verify_simple_data_duplicate(portfolio_weight_series.index, 'portfolio_weight_series.index', trading_day)

            benchmark_weight_series = benchmark_weight_series_dict.get(trading_day)
            verify_type(benchmark_weight_series, 'benchmark_weight_series', pd.Series, trading_day)
            verify_df_not_null(benchmark_weight_series, 'benchmark_weight_series', trading_day)
            verify_simple_data_duplicate(benchmark_weight_series.index, 'benchmark_weight_series.index', trading_day)

            security_sector_series = security_sector_series_dict.get(trading_day)
            verify_type(security_sector_series, 'security_sector_series', pd.Series, trading_day)
            verify_simple_data_duplicate(security_sector_series.index, 'security_sector_series.index', trading_day)

            security_return_series = security_return_series_dict.get(next_trading_day)
            verify_type(security_return_series, 'security_return_series', pd.Series, next_trading_day)
            verify_df_not_null(security_return_series, 'security_return_series', next_trading_day)
            verify_simple_data_duplicate(security_return_series.index, 'security_return_series.index', next_trading_day)

            nav_active_return_series = nav_active_return_dict.get(next_trading_day)
            verify_type(nav_active_return_series, 'nav_active_return_series', pd.Series, next_trading_day)
            verify_df_not_null(nav_active_return_series, 'nav_active_return_series', next_trading_day)

    def _init_data_param(self):
        if not self.data_param:
            return
        self.trading_day_list = self.data_param['trading_day_list']
        self.security_return_series_dict = self.data_param.get('security_return_series_dict')
        self.benchmark_weight_series_dict = self.data_param.get('benchmark_weight_series_dict')
        self.portfolio_weight_series_dict = self.data_param.get('portfolio_weight_series_dict')
        self.security_sector_series_dict = self.data_param.get('security_sector_series_dict')
        self.nav_active_return_dict = self.data_param.get('nav_active_return_dict')
        for i in range(len(self.trading_day_list) - 1):
            trading_day = self.trading_day_list[i]
            next_trading_day = self.trading_day_list[i + 1]
            idx = self.portfolio_weight_series_dict[trading_day].index.union(
                self.benchmark_weight_series_dict[trading_day].index)
            has_data_idx = idx.intersection(self.security_return_series_dict[next_trading_day].index)
            if len(has_data_idx) < len(idx):
                logger.warning('%s losing equity [%s] during intersection with %s equity_reinvest_return_series' % (
                    trading_day, ','.join(map(str, set(idx) - set(has_data_idx))), next_trading_day))
            idx = has_data_idx
            has_data_idx = idx.intersection(self.security_sector_series_dict[trading_day].index)
            if len(has_data_idx) < len(idx):
                logger.warning('%s losing equity [%s] during intersection with %s security_sector_series' % (
                    trading_day, ','.join(map(str, set(idx) - set(has_data_idx))), trading_day))
            idx = has_data_idx
            self.portfolio_weight_series_dict[trading_day] = self.portfolio_weight_series_dict[trading_day].reindex(
                idx).fillna(0.)
            self.benchmark_weight_series_dict[trading_day] = self.benchmark_weight_series_dict[trading_day].reindex(
                idx).fillna(0.)
            self.security_sector_series_dict[trading_day] = self.security_sector_series_dict[trading_day].reindex(idx)
            self.security_return_series_dict[next_trading_day] = self.security_return_series_dict[
                next_trading_day].reindex(idx)
        # 判断
        for trading_day in self.trading_day_list:
            sector_df = self.security_sector_series_dict[trading_day]
            verify_df_not_null(sector_df, 'sector_df', trading_day)
        self.sector_list = pd.concat(self.security_sector_series_dict).unique().tolist()

    @staticmethod
    def _timing_analysis(weight_portfolio_series, weight_benchmark_series, equity_return_series):
        abs_sum_p_weight = weight_portfolio_series.abs().sum()
        abs_sum_b_weight = weight_benchmark_series.abs().sum()
        norm_weight_portfolio_series = weight_portfolio_series / abs_sum_p_weight if \
            abs_sum_p_weight > 0 else weight_portfolio_series
        norm_weight_benchmark_series = weight_benchmark_series / abs_sum_b_weight if \
            abs_sum_b_weight > 0 else weight_benchmark_series

        if abs_sum_p_weight <= 1e-10 and abs_sum_b_weight <= 1e-10:
            R_portfolio, R_benchmark = 0.0, 0.0
        elif abs_sum_p_weight <= 1e-10:
            R_benchmark = norm_weight_benchmark_series.dot(equity_return_series)
            R_portfolio = R_benchmark
        elif abs_sum_b_weight <= 1e-10:
            R_portfolio = norm_weight_portfolio_series.dot(equity_return_series)
            R_benchmark = R_portfolio
        else:
            R_portfolio = norm_weight_portfolio_series.dot(equity_return_series)
            R_benchmark = norm_weight_benchmark_series.dot(equity_return_series)
        portfolio_total_return = R_portfolio * abs_sum_p_weight
        benchmark_total_return = R_benchmark * abs_sum_b_weight
        timing = (abs_sum_p_weight - abs_sum_b_weight) * R_benchmark
        management = abs_sum_p_weight * (R_portfolio - R_benchmark)

        return timing, management, portfolio_total_return, benchmark_total_return, abs_sum_p_weight, abs_sum_b_weight

    @staticmethod
    def _sector_analysis(weight_portfolio_series, weight_benchmark_series,
                         sector_weight_portfolio_series, sector_weight_benchmark_series, equity_return_series):
        abs_sum_p_weight = weight_portfolio_series.abs().sum()
        abs_sum_b_weight = weight_benchmark_series.abs().sum()
        abs_sum_sector_p_weight = sector_weight_portfolio_series.abs().sum()
        abs_sum_sector_b_weight = sector_weight_benchmark_series.abs().sum()

        norm_sector_weight_portfolio_series = sector_weight_portfolio_series / abs_sum_sector_p_weight if \
            abs_sum_sector_p_weight > 0 else sector_weight_portfolio_series
        norm_sector_weight_benchmark_series = sector_weight_benchmark_series / abs_sum_sector_b_weight if \
            abs_sum_sector_b_weight > 0 else sector_weight_benchmark_series

        if abs_sum_sector_p_weight <= 1e-10 and abs_sum_sector_b_weight <= 1e-10:
            R_portfolio, R_benchmark = 0.0, 0.0
        elif abs_sum_sector_p_weight <= 1e-10:
            R_benchmark = norm_sector_weight_benchmark_series.dot(equity_return_series)
            R_portfolio = R_benchmark
            if abs_sum_p_weight <= 1e-10:
                abs_sum_sector_p_weight = abs_sum_sector_b_weight
        elif abs_sum_sector_b_weight <= 1e-10:
            R_portfolio = norm_sector_weight_portfolio_series.dot(equity_return_series)
            R_benchmark = R_portfolio
            if abs_sum_b_weight <= 1e-10:
                abs_sum_sector_b_weight = abs_sum_sector_p_weight
        else:
            R_portfolio = norm_sector_weight_portfolio_series.dot(equity_return_series)
            R_benchmark = norm_sector_weight_benchmark_series.dot(equity_return_series)

        if abs_sum_b_weight >= 1e-10:
            allocation_return = (abs_sum_sector_p_weight - abs_sum_sector_b_weight) * R_benchmark
        else:
            allocation_return = 0.0
        selection_return = abs_sum_sector_b_weight * (R_portfolio - R_benchmark)
        interaction_return = (abs_sum_sector_p_weight - abs_sum_sector_b_weight) * (R_portfolio - R_benchmark)

        allocation = pd.Series(index=['return'], data=[allocation_return])
        selection = pd.Series(index=['return'], data=[selection_return])
        interaction = pd.Series(index=['return'], data=[interaction_return])
        weight = pd.Series(index=['portfolio', 'benchmark'], data=[abs_sum_sector_p_weight, abs_sum_sector_b_weight])
        sector_portfolio_return = sector_weight_portfolio_series.dot(equity_return_series)
        sector_benchmark_return = sector_weight_benchmark_series.dot(equity_return_series)
        ret = pd.Series(index=['portfolio', 'benchmark'], data=[sector_portfolio_return, sector_benchmark_return])

        return allocation, selection, interaction, weight, ret

    def _single_period_attribution(self, weight_portfolio_series, weight_benchmark_series,
                                   equity_return_series, security_sector_series, sector_list):
        abs_sum_p_weight = weight_portfolio_series.abs().sum()
        abs_sum_b_weight = weight_benchmark_series.abs().sum()
        norm_weight_portfolio_series = weight_portfolio_series / abs_sum_p_weight if \
            abs_sum_p_weight > 0 else weight_portfolio_series
        norm_weight_benchmark_series = weight_benchmark_series / abs_sum_b_weight if \
            abs_sum_b_weight > 0 else weight_benchmark_series

        timing, management, portfolio_total_return, benchmark_total_return, portfolio_weight, benchmark_weight = \
            self._timing_analysis(weight_portfolio_series, weight_benchmark_series, equity_return_series)

        total_return = pd.Series(
            index=['portfolio', 'benchmark'], data=[portfolio_total_return, benchmark_total_return])
        total_weight = pd.Series(index=['portfolio', 'benchmark'], data=[portfolio_weight, benchmark_weight])
        realized_active_return_management_df = pd.DataFrame(
            index=sector_list, columns=['sector_allocation', 'equity_selection', 'interaction'])
        active_sector_weight_series = pd.Series(index=sector_list, name='active', dtype=float)
        portfolio_sector_weight_series = pd.Series(index=sector_list, name='portfolio', dtype=float)
        benchmark_sector_weight_series = pd.Series(index=sector_list, name='benchmark', dtype=float)
        active_sector_return_series = pd.Series(index=sector_list, name='active', dtype=float)
        portfolio_sector_return_series = pd.Series(index=sector_list, name='portfolio', dtype=float)
        benchmark_sector_return_series = pd.Series(index=sector_list, name='benchmark', dtype=float)

        for sector in sector_list:
            equity_universe = security_sector_series[security_sector_series == sector].index.tolist()
            if len(equity_universe) == 0:
                allocation = pd.Series(index=['return'], data=[0.])
                selection = pd.Series(index=['return'], data=[0.])
                interaction = pd.Series(index=['return'], data=[0.])
                weight = pd.Series(index=['portfolio', 'benchmark'], data=[0., 0.])
                ret = pd.Series(index=['portfolio', 'benchmark'], data=[0., 0.])
            else:
                sub_weight_portfolio_series = norm_weight_portfolio_series.reindex(equity_universe)
                sub_weight_benchmark_series = norm_weight_benchmark_series.reindex(equity_universe)
                sub_weight_portfolio_series = sub_weight_portfolio_series.reindex(
                    security_sector_series.index).fillna(0.)
                sub_weight_benchmark_series = sub_weight_benchmark_series.reindex(
                    security_sector_series.index).fillna(0.)
                allocation, selection, interaction, weight, ret = self._sector_analysis(
                    norm_weight_portfolio_series, norm_weight_benchmark_series,
                    sub_weight_portfolio_series, sub_weight_benchmark_series, equity_return_series)
            allocation *= abs_sum_p_weight
            selection *= abs_sum_p_weight
            interaction *= abs_sum_p_weight
            weight *= abs_sum_p_weight
            ret *= abs_sum_p_weight

            realized_active_return_management_df.loc[sector] = \
                [allocation['return'], selection['return'], interaction['return']]
            active_sector_weight_series[sector] = weight['portfolio'] - weight['benchmark']
            portfolio_sector_weight_series[sector] = weight['portfolio']
            benchmark_sector_weight_series[sector] = weight['benchmark']
            active_sector_return_series[sector] = ret['portfolio'] - ret['benchmark']
            portfolio_sector_return_series[sector] = ret['portfolio']
            benchmark_sector_return_series[sector] = ret['benchmark']

        sector_weight_df = pd.concat(
            [portfolio_sector_weight_series, benchmark_sector_weight_series, active_sector_weight_series], axis=1)
        sector_return_df = pd.concat(
            [portfolio_sector_return_series, benchmark_sector_return_series, active_sector_return_series], axis=1)
        active_return_dict = {
            'realized_timing': timing,
            'realized_management_df': realized_active_return_management_df
        }

        return active_return_dict, sector_weight_df, total_return, total_weight, sector_return_df

    @staticmethod
    def _accumulate_attribution(active_return, active_weight, total_return_df):
        active_return['realized_accumulated_timing_series'] = \
            accumulate_timing_return(total_return_df, active_return['realized_period_timing_series'])
        active_return['realized_accumulated_timing_value'] = \
            active_return['realized_accumulated_timing_series'].iloc[-1]

        active_return['realized_accumulated_management_df_dict'] = \
            accumulate_management_return(total_return_df, active_return['realized_period_management_df_dict'])
        trading_day_list = sorted(active_return['realized_accumulated_management_df_dict'].keys())
        active_return['realized_accumulated_management_df'] = \
            active_return['realized_accumulated_management_df_dict'][trading_day_list[-1]]

        active_weight['accumulated_sector_weight_series'] = active_weight['sector_weight_df'].reset_index().set_index(
            'date').groupby('sector').mean()
        active_weight['accumulated_total_weight_series'] = active_weight['total_weight_df'].mean(axis=1)

        coef_series = compute_attr_linking_coef(total_return_df.T)
        accum_sector_portfolio_return = (
                active_return['sector_return_df']['portfolio'] * coef_series).reset_index().groupby('sector')[0].mean()
        accum_sector_portfolio_return.name = 'portfolio'
        accum_sector_benchmark_return = \
            (active_return['sector_return_df']['benchmark'] * coef_series).reset_index().groupby('sector')[0].mean()
        accum_sector_benchmark_return.name = 'benchmark'
        accum_sector_active_return = \
            (active_return['sector_return_df']['active'] * coef_series).reset_index().groupby('sector')[0].mean()
        accum_sector_active_return.name = 'active'
        active_return['realized_accumulated_sector_return_df'] = pd.concat([
            accum_sector_portfolio_return, accum_sector_benchmark_return, accum_sector_active_return], axis=1)

        return active_return, active_weight

    def get_attribution_result(self, is_accumulated=True):
        active_return = {
            "realized_period_timing_series": pd.Series(index=self.trading_day_list[1:], dtype=float),
            "realized_period_management_df_dict": {}
        }
        active_weight = {}
        total_return_df = pd.DataFrame(index=['portfolio', 'benchmark'], columns=self.trading_day_list[1:])
        total_weight_df = pd.DataFrame(index=['portfolio', 'benchmark'], columns=self.trading_day_list[:-1])
        sector_weight_df_dict = {}
        sector_return_df_dict = {}
        for i in range(len(self.trading_day_list) - 1):
            trading_day = self.trading_day_list[i]
            next_trading_day = self.trading_day_list[i + 1]
            weight_portfolio_series = self.portfolio_weight_series_dict[trading_day]
            weight_benchmark_series = self.benchmark_weight_series_dict[trading_day]
            equity_return_series = self.security_return_series_dict[next_trading_day]
            security_sector_series = self.security_sector_series_dict[trading_day]
            active_return_dict, sector_weight_df, total_return, total_weight, sector_return_df = \
                self._single_period_attribution(weight_portfolio_series, weight_benchmark_series,
                                                equity_return_series, security_sector_series, self.sector_list)

            total_return_df[next_trading_day] = total_return
            total_weight_df[trading_day] = total_weight
            active_return['realized_period_timing_series'][next_trading_day] = active_return_dict['realized_timing']
            active_return['realized_period_management_df_dict'][next_trading_day] = \
                active_return_dict['realized_management_df']
            sector_weight_df_dict[trading_day] = sector_weight_df
            sector_return_df_dict[next_trading_day] = sector_return_df

        active_weight['sector_weight_df'] = pd.concat(sector_weight_df_dict)
        active_weight['sector_weight_df'].index.names = ['date', 'sector']
        active_weight['total_weight_df'] = total_weight_df

        active_return['sector_return_df'] = pd.concat(sector_return_df_dict)
        active_return['sector_return_df'].index.names = ['date', 'sector']
        self.active_return = active_return
        self.active_weight = active_weight
        self.total_return_df = total_return_df

        if is_accumulated:
            active_return, active_weight = self._accumulate_attribution(active_return, active_weight, total_return_df)

        # brinson_attr_df = []
        # for date in self.trading_day_list[1:]:
        #     brinson_attr = active_return['realized_period_management_df_dict'][date].sum()
        #     brinson_attr.loc['timing'] = active_return['realized_period_timing_series'].loc[date]
        #     brinson_attr_df.append(brinson_attr.to_frame(date))
        # brinson_attr_df = pd.concat(brinson_attr_df, axis=1).T.sort_index()

        return {
            "active_return": active_return,
            "active_weight": active_weight,
            "total_return_df": total_return_df,
            "nav_return_df": pd.DataFrame(self.nav_active_return_dict)
        }


if __name__ == '__main__':
    attr = EquityBrinsonAttribution(
        fund_id='100038', benchmark_id='000300', start_date='20191220', end_date='20210420').get_attribution_result()


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from hbshare.fe.xwq.analysis.utils.const_var import TimeDateFormat
from hbshare.fe.xwq.analysis.utils.timedelta_utils import TimeDateUtil
import numpy as np
import pandas as pd


def get_cal_and_trade_cal(start, end):
    """
    获取交易日期
    """
    cal = HBDB().read_cal(start, end)
    cal = cal.rename(columns={'JYRQ': 'TRADE_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    cal['IS_OPEN'] = cal['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    cal['IS_WEEK_END'] = cal['IS_WEEK_END'].fillna(0).astype(int)
    cal['IS_MONTH_END'] = cal['IS_MONTH_END'].fillna(0).astype(int)
    cal = cal.sort_values('TRADE_DATE')
    trade_cal = cal[cal['IS_OPEN'] == 1]
    trade_cal['RECENT_TRADE_DATE'] = trade_cal['TRADE_DATE']
    trade_cal['PREV_TRADE_DATE'] = trade_cal['TRADE_DATE'].shift(1)
    trade_cal = trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    cal = cal.merge(trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['RECENT_TRADE_DATE'] = cal['RECENT_TRADE_DATE'].fillna(method='ffill')
    cal = cal.merge(trade_cal[['TRADE_DATE', 'PREV_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['PREV_TRADE_DATE'] = cal['PREV_TRADE_DATE'].fillna(method='bfill')
    cal = cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    return cal, trade_cal

def fund_info(date):
    """
    研究对象
    """
    # 正常运行中的普通股票型、偏股混合型、灵活配置型公募基金
    fund = HBDB().read_stock_fund_info()
    fund = fund.rename(columns={'jjdm': 'FUND_CODE', 'jjmc': 'FUND_FULL_NAME', 'jjjc': 'FUND_SHORT_NAME',
                                'clrq': 'BEGIN_DATE', 'zzrq': 'END_DATE', 'ejfl': 'FUND_TYPE', 'kffb': 'OPEN_CLOSE'})
    # 成立距计算日期满1.5年
    date_before = TimeDateUtil.get_previous_date_str(date, TimeDateFormat.YMD.value, TimeDateFormat.YMD.value, 560)
    fund['BEGIN_DATE'] = fund['BEGIN_DATE'].astype(str)
    fund = fund[fund['BEGIN_DATE'] <= date_before]
    fund = fund.sort_values(['FUND_FULL_NAME', 'FUND_CODE']).drop_duplicates('FUND_FULL_NAME')
    # 成立以来股票占基金净资产的比例均值大于60%
    fund_gptzzjb = HBDB().read_fund_gptzzjb_given_codes(fund['FUND_CODE'].unique().tolist())
    fund_gptzzjb = fund_gptzzjb.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'gptzzjb': 'EQUITY_IN_NA'})
    fund_gptzzjb['REPORT_DATE'] = fund_gptzzjb['REPORT_DATE'].astype(str)
    fund_gptzzjb_mean = fund_gptzzjb[['FUND_CODE', 'EQUITY_IN_NA']].groupby('FUND_CODE').mean().reset_index()
    fund_gptzzjb_mean = fund_gptzzjb_mean[fund_gptzzjb_mean['EQUITY_IN_NA'] > 60]
    fund = fund[fund['FUND_CODE'].isin(fund_gptzzjb_mean['FUND_CODE'].unique().tolist())]
    # 近1.5年以来股票占基金净资产的比例均大于50
    fund_gptzzjb = fund_gptzzjb[(fund_gptzzjb['REPORT_DATE'] >= date_before) & (fund_gptzzjb['REPORT_DATE'] <= date)]
    fund_gptzzjb_min = fund_gptzzjb[['FUND_CODE', 'EQUITY_IN_NA']].groupby('FUND_CODE').min().reset_index()
    fund_gptzzjb_min = fund_gptzzjb_min[fund_gptzzjb_min['EQUITY_IN_NA'] > 50]
    fund = fund[fund['FUND_CODE'].isin(fund_gptzzjb_min['FUND_CODE'].unique().tolist())]
    total = len(fund)
    # todo: kffb字段nan值的处理
    fund['OPEN_CLOSE'] = fund['OPEN_CLOSE'].fillna('0')
    fund_overview = pd.DataFrame(index=['普通股票型基金', '偏股混合型基金', '灵活配置型基金'], columns=['开放式基金', '封闭式基金'])
    fund_overview.loc['普通股票型基金', '开放式基金'] = len(fund[(fund['FUND_TYPE'] == '13') & (fund['OPEN_CLOSE'] == '0')])
    fund_overview.loc['普通股票型基金', '封闭式基金'] = len(fund[(fund['FUND_TYPE'] == '13') & (fund['OPEN_CLOSE'] == '1')])
    fund_overview.loc['偏股混合型基金', '开放式基金'] = len(fund[(fund['FUND_TYPE'] == '37') & (fund['OPEN_CLOSE'] == '0')])
    fund_overview.loc['偏股混合型基金', '封闭式基金'] = len(fund[(fund['FUND_TYPE'] == '37') & (fund['OPEN_CLOSE'] == '1')])
    fund_overview.loc['灵活配置型基金', '开放式基金'] = len(fund[(fund['FUND_TYPE'] == '35') & (fund['OPEN_CLOSE'] == '0')])
    fund_overview.loc['灵活配置型基金', '封闭式基金'] = len(fund[(fund['FUND_TYPE'] == '35') & (fund['OPEN_CLOSE'] == '1')])
    return fund, total, fund_overview

def preload_stock_valuation(dates):
    """
    获取股票估值数据
    """
    stock_valuation_list = []
    star_stock_valuation_list = []
    for date in dates:
        print(date)
        stock_valuation_date = HBDB().read_stock_valuation_given_date(date)
        stock_valuation_list.append(stock_valuation_date)
        star_stock_valuation_date = HBDB().read_star_stock_valuation_given_date(date)
        star_stock_valuation_list.append(star_stock_valuation_date)
    stock_valuation = pd.concat(stock_valuation_list)
    stock_valuation.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_valuation.hdf', key='table', mode='w')
    star_stock_valuation = pd.concat(star_stock_valuation_list)
    star_stock_valuation.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/star_stock_valuation.hdf', key='table', mode='w')
    return

def preload_index_cons():
    """
    获取指数成分股数据
    """
    index_dic = {
        'HS300': '3145',
        'ZZ500': '4978',
        'ZZ1000': '39144',
        'ZZ100': '4293',
        'SZ50': '46',
        'SZ100': '16898',
        'SZ180': '30',
        'JCDP': '3472',
        'JCZP': '3473',
        'JCXP': '3474',
        'JCDPJZ': '10055',
        'JCDPCZ': '10054',
        'JCZPJZ': '10057',
        'JCZPCZ': '10056',
        'JCXPJZ': '10059',
        'JCXPCZ': '10058',
    }
    index_cons_list = []
    for index in index_dic.keys():
        print(index)
        index_cons = HBDB().read_index_cons(index_dic[index])
        index_cons['INDEX'] = index
        index_cons_list.append(index_cons)
    index_cons = pd.concat(index_cons_list)
    index_cons.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/index_cons.hdf', key='table', mode='w')
    return


def fund_valuation(date, fund_zc_holding, fund_zc_holding_diff):
    """
    估值分析
    """
    total_zcmv = fund_zc_holding[['FUND_CODE', 'REPORT_DATE', 'MV_IN_NA']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index()
    total_zcmv = total_zcmv[['REPORT_DATE', 'MV_IN_NA']].groupby(['REPORT_DATE']).mean().reset_index()
    # 交易日期
    # cal, trade_cal = get_cal_and_trade_cal('19000101', date)
    # cal.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/cal.hdf', key='table', mode='w')
    # trade_cal.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/trade_cal.hdf', key='table', mode='w')
    cal = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/cal.hdf', key='table')
    trade_cal = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/trade_cal.hdf', key='table')
    fund_zc_holding = fund_zc_holding.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    fund_zc_holding_diff = fund_zc_holding_diff.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    # todo: 港股估值数据暂时取不到
    # preload_stock_valuation(fund_zc_holding['RECENT_TRADE_DATE'].unique().tolist())
    stock_valuation = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_valuation.hdf', key='table')
    star_stock_valuation = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/star_stock_valuation.hdf', key='table')
    stock_valuation = stock_valuation.rename(columns={'SECUCODE': 'TICKER_SYMBOL', 'TRADINGDAY': 'TRADE_DATE', 'TOTALMV': 'MARKET_VALUE', 'PE': 'PE(TTM)', 'PB': 'PB(LF)'})
    star_stock_valuation = star_stock_valuation.rename(columns={'SECUCODE': 'TICKER_SYMBOL', 'TRADINGDAY': 'TRADE_DATE', 'TOTALMV': 'MARKET_VALUE', 'PE': 'PE(TTM)', 'PB': 'PB(LF)'})
    stock_valuation = stock_valuation[['TRADE_DATE', 'TICKER_SYMBOL', 'MARKET_VALUE', 'PE(TTM)', 'PB(LF)']]
    star_stock_valuation = star_stock_valuation[['TRADE_DATE', 'TICKER_SYMBOL', 'MARKET_VALUE', 'PE(TTM)', 'PB(LF)']]
    stock_valuation = pd.concat([stock_valuation, star_stock_valuation]).sort_values(['TRADE_DATE', 'TICKER_SYMBOL'])
    fund_zc_holding = fund_zc_holding.merge(stock_valuation.rename(columns={'TRADE_DATE': 'RECENT_TRADE_DATE'}), on=['RECENT_TRADE_DATE', 'TICKER_SYMBOL'], how='left')
    pe_zc = fund_zc_holding.dropna(subset=['PE(TTM)'])
    pe_zc_weight = pe_zc[['FUND_CODE', 'REPORT_DATE', 'HOLDING_MARKET_VALUE']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index().rename(columns={'HOLDING_MARKET_VALUE': 'TOTAL_HOLDING_MARKET_VALUE'})
    pe_zc = pe_zc.merge(pe_zc_weight, on=['FUND_CODE', 'REPORT_DATE'], how='left')
    pe_zc['WEIGHTED_PE(TTM)'] = pe_zc['PE(TTM)'] * pe_zc['HOLDING_MARKET_VALUE'] / pe_zc['TOTAL_HOLDING_MARKET_VALUE']
    pe_zc = pe_zc[['FUND_CODE', 'REPORT_DATE', 'WEIGHTED_PE(TTM)']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index()
    pb_zc = fund_zc_holding.dropna(subset=['PB(LF)'])
    pb_zc_weight = pb_zc[['FUND_CODE', 'REPORT_DATE', 'HOLDING_MARKET_VALUE']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index().rename(columns={'HOLDING_MARKET_VALUE': 'TOTAL_HOLDING_MARKET_VALUE'})
    pb_zc = pb_zc.merge(pb_zc_weight, on=['FUND_CODE', 'REPORT_DATE'], how='left')
    pb_zc['WEIGHTED_PB(LF)'] = pb_zc['PB(LF)'] * pb_zc['HOLDING_MARKET_VALUE'] / pb_zc['TOTAL_HOLDING_MARKET_VALUE']
    pb_zc = pb_zc[['FUND_CODE', 'REPORT_DATE', 'WEIGHTED_PB(LF)']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index()
    pe_zc['MARK'] = np.nan
    pe_zc.loc[(pe_zc['WEIGHTED_PE(TTM)'] > 0) & (pe_zc['WEIGHTED_PE(TTM)'] <= 30), 'MARK'] = 'low'
    pe_zc.loc[(pe_zc['WEIGHTED_PE(TTM)'] > 30) & (pe_zc['WEIGHTED_PE(TTM)'] <= 50), 'MARK'] = 'middle'
    pe_zc.loc[(pe_zc['WEIGHTED_PE(TTM)'] > 50) | (pe_zc['WEIGHTED_PE(TTM)'] <= 0), 'MARK'] = 'high'
    pe_zc_count = pe_zc[['FUND_CODE', 'REPORT_DATE', 'MARK']].groupby(['REPORT_DATE', 'MARK']).count().reset_index().rename(columns={'FUND_CODE': 'COUNT'})
    pe_zc_count_sum = pe_zc[['FUND_CODE', 'REPORT_DATE']].groupby(['REPORT_DATE']).count().reset_index().rename(columns={'FUND_CODE': 'COUNT_SUM'})
    pe_zc_count = pe_zc_count.merge(pe_zc_count_sum, on=['REPORT_DATE'], how='left')
    pe_zc_count['COUNT_RATIO'] = pe_zc_count['COUNT'] / pe_zc_count['COUNT_SUM']
    pb_zc['MARK'] = np.nan
    pb_zc.loc[(pb_zc['WEIGHTED_PB(LF)'] > 5) | (pb_zc['WEIGHTED_PB(LF)'] <= 0), 'MARK'] = 'high'
    pb_zc['MARK'] = pb_zc['MARK'].fillna('low')
    pb_zc_count = pb_zc[['FUND_CODE', 'REPORT_DATE', 'MARK']].groupby(['REPORT_DATE', 'MARK']).count().reset_index().rename(columns={'FUND_CODE': 'COUNT'})
    pb_zc_count_sum = pb_zc[['FUND_CODE', 'REPORT_DATE']].groupby(['REPORT_DATE']).count().reset_index().rename(columns={'FUND_CODE': 'COUNT_SUM'})
    pb_zc_count = pb_zc_count.merge(pb_zc_count_sum, on=['REPORT_DATE'], how='left')
    pb_zc_count['COUNT_RATIO'] = pb_zc_count['COUNT'] / pb_zc_count['COUNT_SUM']
    # 估值变化
    fund_zc_holding_diff = fund_zc_holding_diff.merge(stock_valuation.rename(columns={'TRADE_DATE': 'RECENT_TRADE_DATE'}), on=['RECENT_TRADE_DATE', 'TICKER_SYMBOL'], how='left')
    pe_zc_diff = fund_zc_holding_diff.dropna(subset=['PE(TTM)'])
    # pe_zc_diff['PE(TTM)_DIFF'] = pe_zc_diff['MV_IN_NA_DIFF'] / 100.0 * pe_zc_diff['PE(TTM)']
    # pe_zc_diff = pe_zc_diff[['FUND_CODE', 'REPORT_DATE', 'PE(TTM)_DIFF']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index()
    # pe_zc_diff = pe_zc_diff[['REPORT_DATE', 'PE(TTM)_DIFF']].groupby(['REPORT_DATE']).mean().reset_index()
    pe_zc_diff['MV_IN_NA_DIFF'] = pe_zc_diff['MV_IN_NA_DIFF'] / 100.0
    pe_zc_diff['ABS_MV_IN_NA_DIFF'] = pe_zc_diff['MV_IN_NA_DIFF'].abs()
    pe_zc_diff_weight = pe_zc_diff[['FUND_CODE', 'REPORT_DATE', 'ABS_MV_IN_NA_DIFF']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index().rename(columns={'ABS_MV_IN_NA_DIFF': 'TOTAL_ABS_MV_IN_NA_DIFF'})
    pe_zc_diff = pe_zc_diff.merge(pe_zc_diff_weight, on=['FUND_CODE', 'REPORT_DATE'], how='left')
    pe_zc_diff['WEIGHTED_PE(TTM)_DIFF'] = pe_zc_diff['PE(TTM)'] * pe_zc_diff['MV_IN_NA_DIFF'] / pe_zc_diff['TOTAL_ABS_MV_IN_NA_DIFF']
    pe_zc_diff = pe_zc_diff[['FUND_CODE', 'REPORT_DATE', 'WEIGHTED_PE(TTM)_DIFF']].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index()
    pe_zc_diff = pe_zc_diff[['REPORT_DATE', 'WEIGHTED_PE(TTM)_DIFF']].groupby(['REPORT_DATE']).mean().reset_index()
    # 核心资产估值溢价：基金重仓股 / 非基金重仓股的PEPB中位数之比
    fund_zc_holding['IS_ZC'] = 1
    fund_holding = stock_valuation[stock_valuation['TRADE_DATE'].isin(fund_zc_holding['RECENT_TRADE_DATE'].unique().tolist())]
    fund_holding = fund_holding.rename(columns={'TRADE_DATE': 'RECENT_TRADE_DATE'}).merge(fund_zc_holding[['RECENT_TRADE_DATE', 'TICKER_SYMBOL', 'IS_ZC']], on=['RECENT_TRADE_DATE', 'TICKER_SYMBOL'], how='left')
    fund_holding['IS_ZC'] = fund_holding['IS_ZC'].fillna(0)
    fund_holding = fund_holding.merge(fund_zc_holding[['RECENT_TRADE_DATE', 'REPORT_DATE']].drop_duplicates(), on=['RECENT_TRADE_DATE'], how='left')
    pe_quantile = fund_holding[['REPORT_DATE', 'IS_ZC', 'PE(TTM)']].groupby(['REPORT_DATE', 'IS_ZC']).quantile().reset_index()
    pe_quantile = pe_quantile.pivot(index='REPORT_DATE', columns='IS_ZC', values='PE(TTM)')
    pe_quantile['RATIO'] = pe_quantile[1] / pe_quantile[0]
    pb_quantile = fund_holding[['REPORT_DATE', 'IS_ZC', 'PB(LF)']].groupby(['REPORT_DATE', 'IS_ZC']).quantile().reset_index()
    pb_quantile = pb_quantile.pivot(index='REPORT_DATE', columns='IS_ZC', values='PB(LF)')
    pb_quantile['RATIO'] = pb_quantile[1] / pb_quantile[0]
    return total_zcmv, pe_zc_count, pb_zc_count, pe_zc_diff, pe_quantile, pb_quantile

def fund_sector(fund_zc_holding):
    """
    板块分析
    """
    # 上市板块
    fund_zc_holding['SECTOR'] = fund_zc_holding['TICKER_SYMBOL'].apply(lambda x: '主板-上海' if x[:2] == '60' else '主板-深圳' if x[:2] == '00' else '创业板-深圳'if x[:2] == '30' else '科创版-上海' if x[:2] == '68' else '主板-香港')
    fund_zc_holding_sector = fund_zc_holding[['REPORT_DATE', 'FUND_CODE', 'SECTOR', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE', 'SECTOR']).count().reset_index().rename(columns={'TICKER_SYMBOL': 'COUNT'})
    fund_zc_holding_sector = fund_zc_holding_sector[['REPORT_DATE', 'SECTOR', 'COUNT']].groupby(['REPORT_DATE', 'SECTOR']).sum().reset_index()
    fund_zc_holding_sector = fund_zc_holding_sector.pivot(index='REPORT_DATE', columns='SECTOR', values='COUNT').fillna(0.0)
    fund_zc_holding_sector = fund_zc_holding_sector.apply(lambda x: x / x.sum(), axis=1)
    return fund_zc_holding_sector

def fund_theme():
    """
    主题分析
    """
    # 大类行业
    fund_theme = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund_theme.hdf', key='table')
    fund_theme = fund_theme.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zclb': 'IS_ZC', 'zblx': 'THEME',
                                            'zgpszb': 'MV_IN_NA', 'jsqbd': 'MV_IN_NA_DIFF', 'tlpj': 'HOMO_AVG'})
    fund_theme['IS_ZC'] = fund_theme['IS_ZC'].astype(int)
    fund_theme = fund_theme.replace({'TMT': 'TMT', 'ZZ': '制造', 'ZQ': '周期', 'DJR': '大金融', 'XF': '消费', 'QT': '其他'})
    fund_theme = fund_theme[fund_theme['IS_ZC'] == 1]
    fund_theme = fund_theme[['REPORT_DATE', 'THEME', 'MV_IN_NA']].groupby(['REPORT_DATE', 'THEME']).mean().reset_index()
    fund_theme = fund_theme.pivot(index='REPORT_DATE', columns='THEME', values='MV_IN_NA').fillna(0.0)
    fund_theme = fund_theme.apply(lambda x: x / x.sum(), axis=1)
    return fund_theme

def fund_industry(fund_zc_holding):
    """
    行业分析
    """
    fund_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund_industry.hdf', key='table')
    fund_industry = fund_industry.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zclb': 'IS_ZC', 'hyhfbz': 'INDUSTRY_VERSION',
                                                  'fldm': 'INDUSTRY_ID', 'flmc': 'INDUSTRY_NAME', 'zzjbl': 'MV_IN_NA', 'hyzjzbjbd': 'MV_IN_NA_DIFF', 'hyzjzbltlpj': 'HOMO_AVG'})
    fund_industry['IS_ZC'] = fund_industry['IS_ZC'].astype(int)
    fund_industry['INDUSTRY_VERSION'] = fund_industry['INDUSTRY_VERSION'].astype(int)
    fund_industry = fund_industry[fund_industry['INDUSTRY_VERSION'] == 2]
    fund_industry = fund_industry[fund_industry['IS_ZC'] == 1]
    fund_industry = fund_industry[['REPORT_DATE', 'INDUSTRY_NAME', 'MV_IN_NA']].groupby(['REPORT_DATE', 'INDUSTRY_NAME']).mean().reset_index()
    fund_industry = fund_industry.pivot(index='REPORT_DATE', columns='INDUSTRY_NAME', values='MV_IN_NA')

    stock_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_industry.hdf', key='table')
    stock_industry = stock_industry.rename(columns={'zqdm': 'TICKER_SYMBOL', 'flmc': 'INDUSTRY_NAME', 'fldm': 'INDUSTRY_ID',
                                                    'fljb': 'INDUSTRY_TYPE', 'hyhfbz': 'INDUSTRY_VERSION',
                                                    'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW'})
    stock_industry['END_DATE'] = stock_industry['END_DATE'].fillna(20990101)
    stock_industry['BEGIN_DATE'] = stock_industry['BEGIN_DATE'].astype(str)
    stock_industry['END_DATE'] = stock_industry['END_DATE'].astype(int).astype(str)
    stock_industry['INDUSTRY_VERSION'] = stock_industry['INDUSTRY_VERSION'].astype(int)
    stock_industry['INDUSTRY_TYPE'] = stock_industry['INDUSTRY_TYPE'].astype(int)
    stock_industry = stock_industry[stock_industry['INDUSTRY_VERSION'] == 2]
    stock_industry = stock_industry[stock_industry['INDUSTRY_TYPE'] == 2]
    fund_zc_holding = fund_zc_holding.merge(stock_industry[['TICKER_SYMBOL', 'INDUSTRY_NAME', 'BEGIN_DATE', 'END_DATE']], on=['TICKER_SYMBOL'], how='left')
    fund_zc_holding = fund_zc_holding[(fund_zc_holding['REPORT_DATE'] >= fund_zc_holding['BEGIN_DATE']) & (fund_zc_holding['REPORT_DATE'] < fund_zc_holding['END_DATE'])]
    fund_zc_holding = fund_zc_holding[['REPORT_DATE', 'FUND_CODE', 'INDUSTRY_NAME', 'MV_IN_NA']].groupby(['REPORT_DATE', 'FUND_CODE', 'INDUSTRY_NAME']).sum().reset_index()
    fund_zc_holding = fund_zc_holding[['REPORT_DATE', 'INDUSTRY_NAME', 'MV_IN_NA']].groupby(['REPORT_DATE', 'INDUSTRY_NAME']).mean().reset_index()
    fund_zc_holding = fund_zc_holding.pivot(index='REPORT_DATE', columns='INDUSTRY_NAME', values='MV_IN_NA')
    return fund_zc_holding

def fund_mv(fund_zc_holding):
    """
    市值分析
    """
    index_cons = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/index_cons.hdf', key='table')
    index_cons = index_cons.rename(columns={'SECUCODE': 'TICKER_SYMBOL', 'ENDDATE': 'END_DATE'})
    index_cons['END_DATE'] = index_cons['END_DATE'].apply(lambda x: x[:10].replace('-', ''))
    index_cons = index_cons[index_cons['INDEX'].isin(['HS300', 'ZZ500', 'ZZ1000'])]
    cal = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/cal.hdf', key='table')
    fund_zc_holding = fund_zc_holding[fund_zc_holding['REPORT_DATE'] >= index_cons['END_DATE'].min()]
    fund_zc_mv = fund_zc_holding.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    fund_zc_mv = fund_zc_mv.merge(index_cons[['TICKER_SYMBOL', 'END_DATE', 'INDEX']].rename(columns={'END_DATE': 'RECENT_TRADE_DATE'}), on=['TICKER_SYMBOL', 'RECENT_TRADE_DATE'], how='left')
    fund_zc_mv['INDEX'] = fund_zc_mv['INDEX'].fillna('非成分股')
    fund_zc_mv_count = fund_zc_mv[['REPORT_DATE', 'FUND_CODE', 'INDEX', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE', 'INDEX']).count().reset_index().rename(columns={'TICKER_SYMBOL': 'COUNT'})
    fund_zc_mv_count = fund_zc_mv_count[['REPORT_DATE', 'INDEX', 'COUNT']].groupby(['REPORT_DATE', 'INDEX']).sum().reset_index()
    fund_zc_mv_count = fund_zc_mv_count.pivot(index='REPORT_DATE', columns='INDEX', values='COUNT').fillna(0.0)
    fund_zc_mv_count = fund_zc_mv_count.apply(lambda x: x / x.sum(), axis=1)

    index_cons = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/index_cons.hdf', key='table')
    index_cons = index_cons.rename(columns={'SECUCODE': 'TICKER_SYMBOL', 'ENDDATE': 'END_DATE'})
    index_cons['END_DATE'] = index_cons['END_DATE'].apply(lambda x: x[:10].replace('-', ''))
    index_cons = index_cons[index_cons['INDEX'].isin(['SZ50', 'ZZ100', 'SZ180', 'HS300'])]
    cal = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/cal.hdf', key='table')
    fund_zc_holding = fund_zc_holding[fund_zc_holding['REPORT_DATE'] >= index_cons['END_DATE'].min()]
    fund_zc_wmv = fund_zc_holding.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    fund_zc_wmv_count = fund_zc_wmv[['REPORT_DATE', 'FUND_CODE', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE']).count().reset_index().rename(columns={'TICKER_SYMBOL': 'COUNT'})
    fund_zc_wmv_count = fund_zc_wmv_count[['REPORT_DATE', 'COUNT']].groupby(['REPORT_DATE']).sum().reset_index()
    for index in ['SZ50', 'ZZ100', 'SZ180', 'HS300']:
        index_cons_index = index_cons[index_cons['INDEX'] == index]
        fund_zc_wmv_index = fund_zc_wmv.merge(index_cons_index[['TICKER_SYMBOL', 'END_DATE', 'INDEX']].rename(columns={'END_DATE': 'RECENT_TRADE_DATE'}), on=['TICKER_SYMBOL', 'RECENT_TRADE_DATE'], how='left')
        fund_zc_wmv_index = fund_zc_wmv_index.dropna(subset=['INDEX'])
        fund_zc_wmv_index_count = fund_zc_wmv_index[['REPORT_DATE', 'FUND_CODE', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE']).count().reset_index().rename(columns={'TICKER_SYMBOL': index})
        fund_zc_wmv_index_count = fund_zc_wmv_index_count[['REPORT_DATE', index]].groupby(['REPORT_DATE']).sum().reset_index()
        fund_zc_wmv_count = fund_zc_wmv_count.merge(fund_zc_wmv_index_count, on=['REPORT_DATE'], how='left')
        fund_zc_wmv_count[index] = fund_zc_wmv_count[index] / fund_zc_wmv_count['COUNT']
    return fund_zc_mv_count, fund_zc_wmv_count

def fund_style(fund_zc_holding):
    """
    风格分析
    """
    index_cons = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/index_cons.hdf', key='table')
    index_cons = index_cons.rename(columns={'SECUCODE': 'TICKER_SYMBOL', 'ENDDATE': 'END_DATE'})
    index_cons['END_DATE'] = index_cons['END_DATE'].apply(lambda x: x[:10].replace('-', ''))
    index_cons = index_cons[index_cons['INDEX'].isin(['JCDPJZ', 'JCDPCZ', 'JCZPJZ', 'JCZPCZ', 'JCXPJZ', 'JCXPCZ'])]
    cal = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/cal.hdf', key='table')
    fund_zc_holding = fund_zc_holding[fund_zc_holding['REPORT_DATE'] >= index_cons['END_DATE'].min()]
    fund_zc_style = fund_zc_holding.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    fund_zc_style = fund_zc_style.merge(index_cons[['TICKER_SYMBOL', 'END_DATE', 'INDEX']].rename(columns={'END_DATE': 'RECENT_TRADE_DATE'}), on=['TICKER_SYMBOL', 'RECENT_TRADE_DATE'], how='left')
    wind_mark = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_style.hdf', key='table')
    wind_mark = wind_mark.rename(columns={'ticker': 'TICKER_SYMBOL', 'trade_date': 'TRADE_DATE', 'type': 'IS_ZC',
                                          'cap_score': 'CAP_SCORE', 'vcg_score': 'VCG_SCORE', 'category': 'CATEGORY'})
    wind_mark = wind_mark[wind_mark['IS_ZC'] == 'main']
    fund_zc_style = fund_zc_style.merge(wind_mark[['TICKER_SYMBOL', 'TRADE_DATE', 'CATEGORY']].rename(columns={'TRADE_DATE': 'RECENT_TRADE_DATE'}), on=['TICKER_SYMBOL', 'RECENT_TRADE_DATE'], how='left')
    fund_zc_style['INDEX'] = fund_zc_style['INDEX'].astype(str)
    fund_zc_style['CATEGORY'] = fund_zc_style['CATEGORY'].astype(str)
    fund_zc_style['MARK'] = fund_zc_style.apply(lambda x: '价值' if x['INDEX'][-2:] == 'JZ' else '成长' if x['INDEX'][-2:] == 'CZ'
                                                          else '价值' if x['CATEGORY'][-3:] == '价值型' else '成长' if x['CATEGORY'][-3:] == '成长型'
                                                          else '平衡', axis=1)
    fund_zc_style_count = fund_zc_style[['REPORT_DATE', 'FUND_CODE', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE']).count().reset_index().rename(columns={'TICKER_SYMBOL': 'COUNT'})
    fund_zc_style_count = fund_zc_style_count[['REPORT_DATE', 'COUNT']].groupby(['REPORT_DATE']).sum().reset_index()
    for m in ['价值', '成长', '平衡']:
        fund_zc_style_mark = fund_zc_style[fund_zc_style['MARK'] == m]
        fund_zc_style_mark_count = fund_zc_style_mark[['REPORT_DATE', 'FUND_CODE', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE']).count().reset_index().rename(columns={'TICKER_SYMBOL': m})
        fund_zc_style_mark_count = fund_zc_style_mark_count[['REPORT_DATE', m]].groupby(['REPORT_DATE']).sum().reset_index()
        fund_zc_style_count = fund_zc_style_count.merge(fund_zc_style_mark_count, on=['REPORT_DATE'], how='left')
        fund_zc_style_count[m] = fund_zc_style_count[m] / fund_zc_style_count['COUNT']

    fund_zc_style['MARK2'] = fund_zc_style.apply(lambda x: '大盘价值' if x['INDEX'] == 'JCDPJZ' else '大盘成长' if x['INDEX'] == 'JCDPCZ'
                                                      else '中盘价值' if x['INDEX'] == 'JCZPJZ' else '中盘成长' if x['INDEX'] == 'JCZPCZ'
                                                      else '小盘价值' if x['INDEX'] == 'JCXPJZ' else '小盘成长' if x['INDEX'] == 'JCXPCZ'
                                                      else '大盘价值' if x['CATEGORY'] == '大盘价值型' else '大盘成长' if x['CATEGORY'] == '大盘成长型' else '大盘平衡' if x['CATEGORY'] == '大盘均衡型'
                                                      else '中盘价值' if x['CATEGORY'] == '中盘价值型' else '中盘成长' if x['CATEGORY'] == '中盘成长型' else '中盘平衡' if x['CATEGORY'] == '中盘均衡型'
                                                      else '小盘价值' if x['CATEGORY'] == '小盘价值型' else '小盘成长' if x['CATEGORY'] == '小盘成长型' else '小盘平衡' if x['CATEGORY'] == '小盘均衡型'
                                                      else np.nan, axis=1)
    fund_zc_style = fund_zc_style.dropna(subset=['MARK2'])
    fund_zc_style2_count = fund_zc_style[['REPORT_DATE', 'FUND_CODE', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE']).count().reset_index().rename(columns={'TICKER_SYMBOL': 'COUNT'})
    fund_zc_style2_count = fund_zc_style2_count[['REPORT_DATE', 'COUNT']].groupby(['REPORT_DATE']).sum().reset_index()
    for m in ['大盘价值', '大盘成长', '大盘平衡', '中盘价值', '中盘成长', '中盘平衡', '小盘价值', '小盘成长', '小盘平衡']:
        fund_zc_style_mark = fund_zc_style[fund_zc_style['MARK2'] == m]
        fund_zc_style_mark_count = fund_zc_style_mark[['REPORT_DATE', 'FUND_CODE', 'TICKER_SYMBOL']].groupby(['REPORT_DATE', 'FUND_CODE']).count().reset_index().rename(columns={'TICKER_SYMBOL': m})
        fund_zc_style_mark_count = fund_zc_style_mark_count[['REPORT_DATE', m]].groupby(['REPORT_DATE']).sum().reset_index()
        fund_zc_style2_count = fund_zc_style2_count.merge(fund_zc_style_mark_count, on=['REPORT_DATE'], how='left')
        fund_zc_style2_count[m] = fund_zc_style2_count[m] / fund_zc_style2_count['COUNT']
    return fund_zc_style_count, fund_zc_style2_count

def fund_barra(fund_zc_holding):
    """
    Barra风格分析
    """
    barra = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/barra_style_exposure.hdf', key='table')
    barra = barra.rename(columns={'ticker': 'TICKER_SYMBOL', 'trade_date': 'TRADE_DATE'})
    factor_list = [factor for factor in list(barra.columns) if factor != 'TICKER_SYMBOL' and factor != 'TRADE_DATE']
    cal = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/cal.hdf', key='table')
    fund_zc_holding = fund_zc_holding[fund_zc_holding['REPORT_DATE'] >= barra['TRADE_DATE'].min()]
    fund_zc_holding = fund_zc_holding.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')
    fund_zc_holding = fund_zc_holding.merge(barra.rename(columns={'TRADE_DATE': 'RECENT_TRADE_DATE'}), on=['RECENT_TRADE_DATE', 'TICKER_SYMBOL'], how='left')
    fund_zc_holding_weight = fund_zc_holding[['REPORT_DATE', 'FUND_CODE', 'MV_IN_NA']].groupby(['REPORT_DATE', 'FUND_CODE']).sum().reset_index().rename(columns={'MV_IN_NA': 'TOTAL_MV_IN_NA'})
    fund_zc_holding = fund_zc_holding.merge(fund_zc_holding_weight, on=['REPORT_DATE', 'FUND_CODE'], how='left')
    fund_zc_holding_barra_list = []
    for factor in factor_list:
        fund_zc_holding['WEIGHTED_{}'.format(factor)] = fund_zc_holding[factor] * fund_zc_holding['MV_IN_NA'] / fund_zc_holding['TOTAL_MV_IN_NA']
        fund_zc_holding_barra = fund_zc_holding[['FUND_CODE', 'REPORT_DATE', 'WEIGHTED_{}'.format(factor)]].groupby(['FUND_CODE', 'REPORT_DATE']).sum().reset_index()
        fund_zc_holding_barra = fund_zc_holding_barra[['REPORT_DATE', 'WEIGHTED_{}'.format(factor)]].groupby(['REPORT_DATE']).mean()
        fund_zc_holding_barra_list.append(fund_zc_holding_barra)
    fund_zc_holding_barra = pd.concat(fund_zc_holding_barra_list, axis=1).reset_index()
    return fund_zc_holding_barra

def stock_holding(fund_zc_holding):
    """
    持股变动
    """
    stock_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_industry.hdf', key='table')
    stock_industry = stock_industry.rename(columns={'zqdm': 'TICKER_SYMBOL', 'flmc': 'INDUSTRY_NAME', 'fldm': 'INDUSTRY_ID',
                                                    'fljb': 'INDUSTRY_TYPE', 'hyhfbz': 'INDUSTRY_VERSION',
                                                    'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW'})
    stock_industry['END_DATE'] = stock_industry['END_DATE'].fillna(20990101)
    stock_industry['BEGIN_DATE'] = stock_industry['BEGIN_DATE'].astype(str)
    stock_industry['END_DATE'] = stock_industry['END_DATE'].astype(int).astype(str)
    stock_industry['INDUSTRY_VERSION'] = stock_industry['INDUSTRY_VERSION'].astype(int)
    stock_industry['INDUSTRY_TYPE'] = stock_industry['INDUSTRY_TYPE'].astype(int)
    stock_industry = stock_industry[stock_industry['INDUSTRY_VERSION'] == 2]
    stock_industry = stock_industry[stock_industry['INDUSTRY_TYPE'] == 1]
    fund_zc_holding = fund_zc_holding.merge(stock_industry[['TICKER_SYMBOL', 'INDUSTRY_ID', 'INDUSTRY_NAME', 'BEGIN_DATE', 'END_DATE']], on=['TICKER_SYMBOL'], how='left')
    fund_zc_holding = fund_zc_holding[(fund_zc_holding['REPORT_DATE'] >= fund_zc_holding['BEGIN_DATE']) & (fund_zc_holding['REPORT_DATE'] < fund_zc_holding['END_DATE'])]
    fund_zc_holding = fund_zc_holding.drop(['BEGIN_DATE', 'END_DATE'], axis=1)
    industry_theme = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/industry_theme.hdf', key='table')
    industry_theme = industry_theme.rename(columns={'fldm': 'INDUSTRY_ID', 'hybh': 'THEME_ID', 'hyzt': 'THEME_NAME',
                                                    'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE'})
    industry_theme['END_DATE'] = industry_theme['END_DATE'].fillna(20990101)
    industry_theme['BEGIN_DATE'] = industry_theme['BEGIN_DATE'].astype(str)
    industry_theme['END_DATE'] = industry_theme['END_DATE'].astype(int).astype(str)
    fund_zc_holding = fund_zc_holding.merge(industry_theme[['INDUSTRY_ID', 'THEME_NAME', 'BEGIN_DATE', 'END_DATE']], on=['INDUSTRY_ID'], how='left')
    fund_zc_holding = fund_zc_holding[(fund_zc_holding['REPORT_DATE'] >= fund_zc_holding['BEGIN_DATE']) & (fund_zc_holding['REPORT_DATE'] < fund_zc_holding['END_DATE'])]
    fund_zc_holding = fund_zc_holding.drop(['BEGIN_DATE', 'END_DATE'], axis=1)
    fund_zc_holding = fund_zc_holding.sort_values(['REPORT_DATE', 'INDUSTRY_NAME', 'MV_IN_NA'], ascending=[True, True, False]).drop_duplicates(['REPORT_DATE', 'INDUSTRY_NAME', 'TICKER_SYMBOL'])
    fund_zc_holding = fund_zc_holding.groupby(['REPORT_DATE', 'INDUSTRY_NAME']).head(3)
    return fund_zc_holding

def fund_holding_analysis(date):
    """
    公募基金持仓分析
    """
    # 基金池
    # fund, total, fund_overview = fund_info(date)
    # fund.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund.hdf', key='table', mode='w')
    # fund = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund.hdf', key='table')
    # 基金持仓
    # fund_holding = HBDB().read_fund_holding_given_codes(fund['FUND_CODE'].unique().tolist())
    # fund_holding.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund_holding.hdf', key='table', mode='w')
    fund_holding = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund_holding.hdf', key='table')
    # fund_holding_diff = HBDB().read_fund_holding_diff_given_codes(fund['FUND_CODE'].unique().tolist())
    # fund_holding_diff.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund_holdingHBDB().read_fund_holding_given_codes(fund['FUND_CODE'].unique().tolist())_diff.hdf', key='table', mode='w')
    fund_holding_diff = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/fund_holding_diff.hdf', key='table')
    fund_holding = fund_holding.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zqdm': 'TICKER_SYMBOL', 'zqmc': 'SEC_SHORT_NAME',
                                                'ccsz': 'HOLDING_MARKET_VALUE', 'ccsl': 'HOLDING_AMOUNT', 'zjbl': 'MV_IN_NA'})
    fund_holding['REPORT_DATE'] = fund_holding['REPORT_DATE'].astype(str)
    fund_holding_diff = fund_holding_diff.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'zqdm': 'TICKER_SYMBOL', 'zqmc': 'SEC_SHORT_NAME',
                                                          'zclb': 'IS_ZC', 'zgblbd': 'MV_IN_NA_DIFF', 'sfsqzcg': 'IS_LAST_ZC'})
    fund_holding_diff['REPORT_DATE'] = fund_holding_diff['REPORT_DATE'].astype(str)
    fund_holding_diff['IS_ZC'] = fund_holding_diff['IS_ZC'].astype(int)
    fund_holding_diff['IS_LAST_ZC'] = fund_holding_diff['IS_LAST_ZC'].astype(int)
    # 基金重仓
    fund_zc_holding = fund_holding.sort_values(['FUND_CODE', 'REPORT_DATE', 'MV_IN_NA'], ascending=[True, True, False]).groupby(['FUND_CODE', 'REPORT_DATE']).head(10)
    fund_zc_holding_diff = fund_holding_diff[fund_holding_diff['IS_ZC'] == 1]
    # 估值变化
    # fund_valuation(date, fund_zc_holding, fund_zc_holding_diff)
    # 行业变化
    fund_theme()
    fund_industry(fund_zc_holding)
    fund_mv(fund_zc_holding)
    fund_style(fund_zc_holding)
    fund_barra(fund_zc_holding)
    stock_holding(fund_zc_holding)
    return

if __name__ == "__main__":
    analysis_date = '20211231'
    fund_holding_analysis(analysis_date)





"""
宽基指数的行业收益拆分
"""
import hbshare as hbs
import pandas as pd
import datetime
from hbshare.fe.common.util.config import industry_name

# from plotly.graph_objs import Scatter, Layout
import plotly
from plotly.offline import plot as plot_ly
import plotly.graph_objs as go

plotly.offline.init_notebook_mode(connected=True)


class IndexSectorReturnAttr:
    def __init__(self, start_date, end_date, benchmark_id):
        self.start_date = start_date
        self.end_date = end_date
        self.benchmark_id = benchmark_id
        self._load_data()

    def _load_data(self):
        trading_day_list = self._load_calendar()
        benchmark_weight_series_dict = self._load_benchmark_weight(trading_day_list)
        security_return_series_dict = self._load_security_return_series_dict(benchmark_weight_series_dict)
        security_sector_series_dict = self._load_security_sector(benchmark_weight_series_dict)

        data_param = {"trading_day_list": trading_day_list,
                      "benchmark_weight_series_dict": benchmark_weight_series_dict,
                      "security_return_series_dict": security_return_series_dict,
                      "security_sector_series_dict": security_sector_series_dict}

        self.data_param = data_param

    def _load_calendar(self):
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        trading_day_list = df[df['isMonthEnd'] == 1]['calendarDate'].tolist()

        return trading_day_list

    def _load_benchmark_weight(self, trading_day_list):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and SecuCode = '{}'".format(
            self.benchmark_id)
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code = index_info.set_index('SECUCODE').loc[self.benchmark_id, 'INNERCODE']
        benchmark_weight_series_dict = dict()
        for date in trading_day_list:
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "consTickerSymbol", "ENDDATE": "effDate", "WEIGHT": "weight"})
            benchmark_weight_series_dict[date] = weight_df.set_index(
                'consTickerSymbol')['weight'] / 100.

        return benchmark_weight_series_dict

    def _load_security_return_series_dict(self, benchmark_weight_series_dict):
        ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)

        all_data = []
        for trading_day in sorted(benchmark_weight_series_dict.keys()) + [self.end_date]:
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode " \
                         "and rownum=1) SecuCode, b.TradingDay, b.BackwardPrice FROM hsjy_gg.QT_PerformanceData b " \
                         "where b.TradingDay = to_date('{}', 'yyyymmdd')".format(trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            data_main = pd.DataFrame(res['data'])

            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode " \
                         "and rownum=1) SecuCode, b.TradingDay, b.BackwardPrice FROM " \
                         "hsjy_gg.LC_STIBPerformanceData b " \
                         "where b.TradingDay = to_date('{}', 'yyyymmdd')".format(trading_day)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            data_stib = pd.DataFrame(res['data'])

            all_data.append(pd.concat([data_main, data_stib]))

        data = pd.concat(all_data)
        data.rename(
            columns={"SECUCODE": "ticker", "BACKWARDPRICE": "adj_price", "TRADINGDAY": "trade_date"}, inplace=True)
        data = data[data['ticker'].isin(ticker_list)]
        data['trade_date'] = data['trade_date'].apply(
            lambda x: datetime.datetime.strptime(x, '%Y-%m-%d %H:%M:%S').strftime('%Y%m%d'))
        pivot_df = pd.pivot_table(data, index='trade_date', columns='ticker', values='adj_price').sort_index()
        pct_chg = pivot_df.pct_change().shift(-1).dropna(how='all')
        # process
        security_return_series_dict = dict()
        for date in pct_chg.index.tolist():
            security_return_series_dict[date] = pct_chg.loc[date].dropna()

        return security_return_series_dict

    @staticmethod
    def _load_security_sector(benchmark_weight_series_dict):
        trading_day_list = sorted(benchmark_weight_series_dict.keys())

        ticker_list = list(pd.DataFrame.from_dict(benchmark_weight_series_dict).index)
        n = 300
        group_ticker_list = [ticker_list[i: i + n] for i in range(0, len(ticker_list), n)]
        cols_list = ['ticker'] + [x.lower() for x in industry_name['sw'].values()]

        security_sector_series_dict = dict()
        for date in trading_day_list:
            all_data = []
            for group_ticker in group_ticker_list:
                sql_script = "SELECT {} FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}' and " \
                             "ticker in ({})".format(','.join(cols_list), date,
                                                     ','.join("'{0}'".format(x) for x in group_ticker))
                res = hbs.db_data_query('alluser', sql_script, page_size=5000)
                data = pd.DataFrame(res['data'])
                all_data.append(data)

            factor_exposure = pd.concat(all_data).set_index('ticker')
            reverse_ind = dict([(value.lower(), key) for (key, value) in industry_name['sw'].items()])
            ind_exposure = factor_exposure[reverse_ind.keys()].rename(columns=reverse_ind)
            ind_exposure = ind_exposure.reset_index().melt(
                id_vars=['ticker'], value_vars=list(reverse_ind.values()), var_name='industryName1', value_name='sign')
            ind_exposure = ind_exposure[ind_exposure['sign'] == '1']
            security_sector_series_dict[date] = ind_exposure.set_index(
                'ticker').reindex(ticker_list)['industryName1'].dropna()

        return security_sector_series_dict

    def get_construct_result(self):
        trading_day_list = self.data_param.get('trading_day_list')
        benchmark_weight_series_dict = self.data_param.get('benchmark_weight_series_dict')
        security_return_series_dict = self.data_param.get('security_return_series_dict')
        security_sector_series_dict = self.data_param.get('security_sector_series_dict')

        ind_attr_list = []
        for trading_day in trading_day_list:
            benchmark_weight_series = benchmark_weight_series_dict[trading_day]
            security_return_series = security_return_series_dict[trading_day]
            security_sector_series = security_sector_series_dict[trading_day]

            idx = benchmark_weight_series.index.intersection(security_return_series.index).intersection(
                security_sector_series.index)

            benchmark_weight_series = benchmark_weight_series.reindex(idx)
            security_return_series = security_return_series.reindex(idx)
            security_sector_series = security_sector_series.reindex(idx)

            df = benchmark_weight_series.to_frame('weight').merge(
                security_return_series.to_frame('return'), left_index=True, right_index=True).merge(
                security_sector_series.to_frame('industry'), left_index=True, right_index=True)

            ind_attr = df.groupby('industry').apply(lambda x: x['weight'].dot(x['return'])).to_frame(trading_day)

            ind_attr_list.append(ind_attr)

        attr_df = pd.concat(ind_attr_list, axis=1)
        period_ind_attr = (1 + attr_df.T).prod() - 1

        return period_ind_attr


if __name__ == '__main__':
    ind_attr1 = IndexSectorReturnAttr('20201220', '20211028', '000300').get_construct_result()
    ind_attr2 = IndexSectorReturnAttr('20201220', '20211028', '000905').get_construct_result()
    ind_attr3 = IndexSectorReturnAttr('20201220', '20211028', '000852').get_construct_result()

    ind_attr_all = ind_attr1.to_frame('沪深300').merge(
        ind_attr2.to_frame('中证500'), left_index=True, right_index=True, how='outer').merge(
        ind_attr3.to_frame('中证1000'), left_index=True, right_index=True, how='outer').fillna(0.).sort_values(by='中证500')

    trace0 = go.Bar(x=ind_attr_all.index, y=ind_attr_all['沪深300'], name='沪深300', marker=dict(color='rgb(216, 0, 18)'))
    trace1 = go.Bar(x=ind_attr_all.index, y=ind_attr_all['中证500'], name='中证500', marker=dict(color='rgb(60, 127, 175)'))
    trace2 = go.Bar(x=ind_attr_all.index, y=ind_attr_all['中证1000'], name='中证1000', marker=dict(color='rgb(102, 102, 102)'))

    data_py = [trace0, trace1, trace2]

    plot_ly(data_py, filename='D:\\市场微观结构图\\今年以来宽基指数的行业贡献.html')


from __future__ import division
from ast import If
from re import I
import time
from unicodedata import name
import pandas as pd
from scipy.fft import ifft
import sys,os


import hbshare as hbs
from hbshare.db.simu import cons as ct
"""
@author: jingyang.dai
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: corp.py
@time: 2020/9/18 9:59
"""



def get_prod_valuation_by_jjdm_gzrq(jjdm, gzrq, retry_count=3, pause=0.01, timeout=10):
    """
            根据基金代码、估值日期查询估值数据
        :param jjdm: 基金代码
        :param gzrq: 估值日期
        :return: jjdm 基金代码
                gzrq 估值日期
                kmdm 科目代码
                kmmc 科目名称   
                bz 币种
                hl 汇率
                sl 数量
                dwcb 单位成本
                cb 成本
                cbzjz 成本占净值 %
                hqssj 行情收市价(市价)
                sz 市值
                szzjz 市值占净值 %
                gzzz 估值增值
                tpxx 停牌信息
                qyxx 权益信息
        """
    api = hbs.hb_api()
    for _ in range(retry_count):
        time.sleep(pause)
        ct._write_console()
        # 测试环境
        # url = ct.HOWBUY_SIMU_GET_VALUATION_LIST_BY_JJDM_GZRQ % (
        #     ct.P_TYPE['http'], '192.168.220.107:8180', gzrq, jjdm)
        domain = ct.DOMAINS['ams']
        if hbs.is_prod_env():
            domain = 'ams.inner.howbuy.com'
        url = ct.HOWBUY_SIMU_GET_VALUATION_LIST_BY_JJDM_GZRQ % (
            ct.P_TYPE['http'], domain, gzrq, jjdm)

        org_js = api.query(url)
        status_code = str(org_js['code'])
        if status_code != '0000':
            status = str(org_js['desc'])
            raise ValueError(status)

        if 'body' not in org_js:
            status = "未查询到估值信息"
            raise ValueError(status)

        data = org_js['body']
        prod_df = pd.DataFrame(data, columns=ct.HOWBUY_SIMU_VALUATION)
        # corp_df['tsname'] = corp_df['tsname'].astype(str)
        # corp_df['tsshortName'] = corp_df['tsshortName'].str.replace("#", "").str.replace("$", "").astype(str)
        # corp_df['tscode'] = corp_df['tscode'].astype(str)

        return prod_df
    raise IOError(ct.NETWORK_URL_ERROR_MSG)


if __name__ == "__main__":
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data = hbs.get_prod_valuation_by_jjdm_gzrq("S29494", "20150723")
    print(data)



ManagerDict = dict(
	RYXB="人员性别",
	RYXM="人员姓名",
	RYXL="人员学历",
	CYNX="从业年限",
	RZRQ="入职日期",
	LRRQ="离任日期",
	RYZT="人员状态",
	JJDM="基金代码",
	GGRQ="公告日期",
	RYDM="人员代码",
)


FundInfoDict = dict(
	FGSX="风格属性",
	TZFG="投资风格",
	JJJC="基金简称",
	JJXZ="基金性质",
	CLRQ="成立日期",
	JJFL="基金分类",
	GGRQ="公告日期",
	JJDM="基金代码",
	CPFL="产品分类",    # 1为固收 2为公募 4为私募
	ZZRQ="终止日期",
	JXFL=""
)


FundNetValueDict = dict(
	JJDM="基金代码",
	JZRQ="净值日期",
	LJJZ="累计净值",
	JJJZ="基金净值",
)


PublicFundFirstCategory = {
	"1": "股票型",
	"3": "混合型",
	"7": "货币型",
	"9": "QDII",
	"2": "债券型",
	"f": "FOF",
	"4": "另类投资"
}


PublicFundSecondCategory = {
	"13": "普通股票型",
	"14": "股票型",
	"15": "增强指数型",
	"35": "灵活配置型",
	"16": "被动指数型",
	"37": "偏股混合型",
	"36": "混合型",
	"38": "偏债混合型",
	"91": "股票型QDII",
	"93": "混合型QDII",
	"f3": "债券型FOF",
	"94": "债券型QDII",
	"42": "商品型",
	"f1": "股票型FOF",
	"95": "另类型QDII",
	"41": "股票多空",
	"21": "被动指数型债券",
	"f4": "货币型FOF",
	"23": "混合债券型一级",
	"22": "短期纯债型",
	"25": "增强指数型债券",
	"26": "债券型",
	"43": "REITs",
	"27": "中长期纯债型",
	"24": "混合债券型二级",
	"f2": "混合型FOF",
	"f5": "另类投资FOF"
}




